@@ -1,23 +1,17 @@
 #!/bin/sh
-rm -fr .etc
-mkdir .etc
+# This file is part of Firejail project
+# Copyright (C) 2014-2020 Firejail Authors
+# License GPL v2
 
-for file in etc/*.profile etc/*.inc etc/*.net;
-do
-	sed "s;/etc/firejail;$1/firejail;g" $file > .$file
-done
-
-if [ "x$2" = "xyes" ]
-then
 sed -i -e '
 1i# Workaround for systems where common UNIX utilities are symlinks to busybox.\
 # If this is not your case you can remove --enable-busybox-workaround from\
 # ./configure options, for added security.\
+noblacklist \${PATH}/busybox\
+noblacklist \${PATH}/crontab\
 noblacklist \${PATH}/mount\
-noblacklist \${PATH}/umount\
+noblacklist \${PATH}/nc\
 noblacklist \${PATH}/su\
 noblacklist \${PATH}/sudo\
-noblacklist \${PATH}/nc\
-noblacklist \${PATH}/crontab\
-' .etc/disable-common.inc
-fi
+noblacklist \${PATH}/umount\
+' "$1"
