@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -20,7 +20,7 @@
 #include "faudit.h"
 
 void pid_test(void) {
-	char *kern_proc[] = {
+	static char *kern_proc[] = {
 		"kthreadd",
 		"ksoftirqd",
 		"kworker",
