@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -24,17 +24,73 @@
 #include <errno.h>
 #include <pwd.h>
 
+#if HAVE_SELINUX
+#include <sys/stat.h>
+#include <sys/types.h>
+
+#include <selinux/context.h>
+#include <selinux/label.h>
+#include <selinux/selinux.h>
+#endif
+
 int arg_quiet = 0;
+int arg_debug = 0;
 static int arg_follow_link = 0;
 
-#define COPY_LIMIT (500 * 1024 *1024)
+static unsigned long copy_limit = 500 * 1024 * 1024; // 500 MB
+static unsigned long size_cnt = 0;
 static int size_limit_reached = 0;
 static unsigned file_cnt = 0;
-static unsigned size_cnt = 0;
 
 static char *outpath = NULL;
 static char *inpath = NULL;
 
+#if HAVE_SELINUX
+static struct selabel_handle *label_hnd = NULL;
+static int selinux_enabled = -1;
+#endif
+
+// copy from firejail/selinux.c
+static void selinux_relabel_path(const char *path, const char *inside_path)
+{
+#if HAVE_SELINUX
+        char procfs_path[64];
+	char *fcon = NULL;
+	int fd;
+	struct stat st;
+
+	if (selinux_enabled == -1)
+		selinux_enabled = is_selinux_enabled();
+
+	if (!selinux_enabled)
+		return;
+
+	if (!label_hnd)
+		label_hnd = selabel_open(SELABEL_CTX_FILE, NULL, 0);
+
+	/* Open the file as O_PATH, to pin it while we determine and adjust the label */
+        fd = open(path, O_NOFOLLOW|O_CLOEXEC|O_PATH);
+	if (fd < 0)
+		return;
+	if (fstat(fd, &st) < 0)
+		goto close;
+
+        if (selabel_lookup_raw(label_hnd, &fcon, inside_path, st.st_mode)  == 0) {
+		sprintf(procfs_path, "/proc/self/fd/%i", fd);
+		if (arg_debug)
+			printf("Relabeling %s as %s (%s)\n", path, inside_path, fcon);
+
+		setfilecon_raw(procfs_path, fcon);
+        }
+	freecon(fcon);
+ close:
+	close(fd);
+#else
+	(void) path;
+	(void) inside_path;
+#endif
+}
+
 // modified version of the function from util.c
 static void copy_file(const char *srcname, const char *destname, mode_t mode, uid_t uid, gid_t gid) {
 	assert(srcname);
@@ -55,7 +111,7 @@ static void copy_file(const char *srcname, const char *destname, mode_t mode, ui
 	}
 
 	// open destination
-	int dst = open(destname, O_CREAT|O_WRONLY|O_TRUNC, 0755);
+	int dst = open(destname, O_CREAT|O_WRONLY|O_TRUNC, S_IRUSR | S_IWUSR);
 	if (dst < 0) {
 		if (!arg_quiet)
 			fprintf(stderr, "Warning fcopy: cannot open %s, file not copied\n", destname);
@@ -76,7 +132,8 @@ static void copy_file(const char *srcname, const char *destname, mode_t mode, ui
 			done += rv;
 		}
 	}
-	fflush(0);
+	if (len < 0)
+		goto errexit;
 
 	if (fchown(dst, uid, gid) == -1)
 		goto errexit;
@@ -86,6 +143,8 @@ static void copy_file(const char *srcname, const char *destname, mode_t mode, ui
 	close(src);
 	close(dst);
 
+	selinux_relabel_path(destname, srcname);
+
 	return;
 
 errexit:
@@ -121,7 +180,7 @@ void copy_link(const char *target, const char *linkpath, mode_t mode, uid_t uid,
 
 	// if the link is already there, don't create it
 	struct stat s;
-	if (stat(linkpath, &s) == 0)
+	if (lstat(linkpath, &s) == 0)
 	       return;
 
 	char *rp = realpath(target, NULL);
@@ -184,8 +243,8 @@ static int fs_copydir(const char *infname, const struct stat *st, int ftype, str
 	mode_t mode = s.st_mode;
 
 	// recalculate size
-	if ((s.st_size + size_cnt) > COPY_LIMIT) {
-		fprintf(stderr, "Error fcopy: size limit of %dMB reached\n", (COPY_LIMIT / 1024) / 1024);
+	if ((s.st_size + size_cnt) > copy_limit) {
+		fprintf(stderr, "Error fcopy: size limit of %lu MB reached\n", (copy_limit / 1024) / 1024);
 		size_limit_reached = 1;
 		free(outfname);
 		return 0;
@@ -330,6 +389,9 @@ int main(int argc, char **argv) {
 	char *quiet = getenv("FIREJAIL_QUIET");
 	if (quiet && strcmp(quiet, "yes") == 0)
 		arg_quiet = 1;
+	char *debug = getenv("FIREJAIL_DEBUG");
+	if (debug && strcmp(debug, "yes") == 0)
+		arg_debug = 1;
 
 	char *src;
 	char *dest;
@@ -350,25 +412,21 @@ int main(int argc, char **argv) {
 		exit(1);
 	}
 
-	// trim trailing chars
-	if (src[strlen(src) - 1] == '/')
-		src[strlen(src) - 1] = '\0';
-	if (dest[strlen(dest) - 1] == '/')
-		dest[strlen(dest) - 1] = '\0';
+	warn_dumpable();
 
 	// check the two files; remove ending /
-	int len = strlen(src);
-	if (src[len - 1] == '/')
-		src[len - 1] = '\0';
-	if (strcspn(src, "\\*&!?\"'<>%^(){}[];,") != (size_t)len) {
+	size_t len = strlen(src);
+	while (len > 1 && src[len - 1] == '/')
+		src[--len] = '\0';
+	if (strcspn(src, "\\*&!?\"'<>%^(){}[];,") != len) {
 		fprintf(stderr, "Error fcopy: invalid source file name %s\n", src);
 		exit(1);
 	}
 
 	len = strlen(dest);
-	if (dest[len - 1] == '/')
-		dest[len - 1] = '\0';
-	if (strcspn(dest, "\\*&!?\"'<>%^(){}[];,~") != (size_t)len) {
+	while (len > 1 && dest[len - 1] == '/')
+		dest[--len] = '\0';
+	if (strcspn(dest, "\\*&!?\"'<>%^(){}[];,~") != len) {
 		fprintf(stderr, "Error fcopy: invalid dest file name %s\n", dest);
 		exit(1);
 	}
@@ -384,6 +442,14 @@ int main(int argc, char **argv) {
 		exit(1);
 	}
 
+	// extract copy limit size from env variable, if any
+	char *cl = getenv("FIREJAIL_FILE_COPY_LIMIT");
+	if (cl) {
+		copy_limit = strtoul(cl, NULL, 10) * 1024 * 1024;
+		if (arg_debug)
+			printf("file copy limit %lu bytes\n", copy_limit);
+	}
+
 	// copy files
 	if ((arg_follow_link ? stat : lstat)(src, &s) == -1) {
 		fprintf(stderr, "Error fcopy: src %s: %s\n", src, strerror(errno));
