@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -18,6 +18,8 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 #include "firejail.h"
+#include "../include/seccomp.h"
+#include "../include/syscall.h"
 #include <sys/stat.h>
 #include <linux/loop.h>
 
@@ -31,6 +33,8 @@ char *xpra_extra_params = "";
 char *xvfb_screen = "800x600x24";
 char *xvfb_extra_params = "";
 char *netfilter_default = NULL;
+unsigned long join_timeout = 5000000; // microseconds
+char *config_seccomp_error_action_str = "EPERM";
 
 int checkcfg(int val) {
 	assert(val < CFG_MAX);
@@ -50,6 +54,8 @@ int checkcfg(int val) {
 		cfg_val[CFG_DISABLE_MNT] = 0;
 		cfg_val[CFG_ARP_PROBES] = DEFAULT_ARP_PROBES;
 		cfg_val[CFG_XPRA_ATTACH] = 0;
+		cfg_val[CFG_SECCOMP_ERROR_ACTION] = -1;
+		cfg_val[CFG_BROWSER_ALLOW_DRM] = 0;
 
 		// open configuration file
 		const char *fname = SYSCONFDIR "/firejail.config";
@@ -213,6 +219,27 @@ int checkcfg(int val) {
 				if (setenv("FIREJAIL_FILE_COPY_LIMIT", ptr + 16, 1) == -1)
 					errExit("setenv");
 			}
+
+			// timeout for join option
+			else if (strncmp(ptr, "join-timeout ", 13) == 0)
+				join_timeout = strtoul(ptr + 13, NULL, 10) * 1000000; // seconds to microseconds
+
+			// seccomp error action
+			else if (strncmp(ptr, "seccomp-error-action ", 21) == 0) {
+				if (strcmp(ptr + 21, "kill") == 0)
+					cfg_val[CFG_SECCOMP_ERROR_ACTION] = SECCOMP_RET_KILL;
+				else if (strcmp(ptr + 21, "log") == 0)
+					cfg_val[CFG_SECCOMP_ERROR_ACTION] = SECCOMP_RET_LOG;
+				else {
+					cfg_val[CFG_SECCOMP_ERROR_ACTION] = errno_find_name(ptr + 21);
+					if (cfg_val[CFG_SECCOMP_ERROR_ACTION] == -1)
+						errExit("seccomp-error-action: unknown errno");
+				}
+				config_seccomp_error_action_str = strdup(ptr + 21);
+				if (!config_seccomp_error_action_str)
+					errExit("strdup");
+			}
+
 			else
 				goto errout;
 
@@ -268,6 +295,14 @@ void print_compiletime_support(void) {
 #endif
 		);
 
+	printf("\t- D-BUS proxy support is %s\n",
+#ifdef HAVE_DBUSPROXY
+		"enabled"
+#else
+		"disabled"
+#endif
+		);
+
 	printf("\t- file and directory whitelisting support is %s\n",
 #ifdef HAVE_WHITELIST
 		"enabled"
@@ -316,8 +351,16 @@ void print_compiletime_support(void) {
 #endif
 		);
 
-	printf("\t- seccomp-bpf support is %s\n",
-#ifdef HAVE_SECCOMP
+	printf("\t- private-cache and tmpfs as user %s\n",
+#ifdef HAVE_USERTMPFS
+		"enabled"
+#else
+		"disabled"
+#endif
+		);
+
+	printf("\t- SELinux support is %s\n",
+#ifdef HAVE_SELINUX
 		"enabled"
 #else
 		"disabled"
