@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -185,12 +185,12 @@ static void duplicate(char *fname) {
 	// add to private-lib list
 	if (cfg.bin_private_lib == NULL) {
 		if (asprintf(&cfg.bin_private_lib, "%s,%s",fname, full_path) == -1)
-			errExit("asprinf");
+			errExit("asprintf");
 	}
 	else {
 		char *tmp;
 		if (asprintf(&tmp, "%s,%s,%s", cfg.bin_private_lib, fname, full_path) == -1)
-			errExit("asprinf");
+			errExit("asprintf");
 		free(cfg.bin_private_lib);
 		cfg.bin_private_lib = tmp;
 	}
@@ -309,5 +309,6 @@ void fs_private_bin_list(void) {
 		}
 		i++;
 	}
+	selinux_relabel_path(RUN_BIN_DIR, "/bin");
 	fmessage("%d %s installed in %0.2f ms\n", prog_cnt, (prog_cnt == 1)? "program": "programs", timetrace_end());
 }
