@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -158,9 +158,6 @@ static void create_link(const char *oldpath, const char *newpath) {
 		fprintf(stderr, "Error: cannot create %s device\n", newpath);
 		exit(1);
 	}
-
-	if (chown(newpath, 0, 0) < 0) {;}
-
 	fs_logger2("create", newpath);
 	return;
 }
@@ -168,6 +165,7 @@ static void create_link(const char *oldpath, const char *newpath) {
 static void empty_dev_shm(void) {
 	// create an empty /dev/shm directory
 	mkdir_attr("/dev/shm", 01777, 0, 0);
+	selinux_relabel_path("/dev/shm", "/dev/shm");
 	fs_logger("mkdir /dev/shm");
 	fs_logger("create /dev/shm");
 }
@@ -246,6 +244,8 @@ void fs_private_dev(void){
 				errExit("mounting /dev/log");
 			fs_logger("clone /dev/log");
 		}
+		if (mount(RUN_RO_FILE, RUN_DEVLOG_FILE, "none", MS_BIND, "mode=400,gid=0") < 0)
+			errExit("blacklisting " RUN_DEVLOG_FILE);
 	}
 
 	// bring forward the current /dev/shm directory if necessary
@@ -277,10 +277,13 @@ void fs_private_dev(void){
 	// pseudo-terminal
 	mkdir_attr("/dev/pts", 0755, 0, 0);
 	fs_logger("mkdir /dev/pts");
+	selinux_relabel_path("/dev/pts", "/dev/pts");
 	fs_logger("create /dev/pts");
 	create_char_dev("/dev/pts/ptmx", 0666, 5, 2); //"mknod -m 666 /dev/pts/ptmx c 5 2");
+	selinux_relabel_path("/dev/pts/ptmx", "/dev/pts/ptmx");
 	fs_logger("mknod /dev/pts/ptmx");
 	create_link("/dev/pts/ptmx", "/dev/ptmx");
+	selinux_relabel_path("/dev/ptmx", "/dev/ptmx");
 
 // code before github issue #351
 	// mount -vt devpts -o newinstance -o ptmxmode=0666 devpts //dev/pts
@@ -299,12 +302,10 @@ void fs_private_dev(void){
 	fs_logger("clone /dev/pts");
 
 	// stdin, stdout, stderr
-#if 0
 	create_link("/proc/self/fd", "/dev/fd");
 	create_link("/proc/self/fd/0", "/dev/stdin");
 	create_link("/proc/self/fd/1", "/dev/stdout");
 	create_link("/proc/self/fd/2", "/dev/stderr");
-#endif
 
 	// symlinks for DVD/CD players
 	if (stat("/dev/sr0", &s) == 0) {
