@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -20,7 +20,6 @@
 #include "firejail.h"
 #include <sys/mount.h>
 #include <linux/limits.h>
-#include <glob.h>
 #include <dirent.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -60,6 +59,7 @@ static void skel(const char *homedir, uid_t u, gid_t g) {
 			touch_file_as_user(fname, 0644);
 			fs_logger2("touch", fname);
 		}
+		selinux_relabel_path(fname, fname);
 		free(fname);
 	}
 	// csh
@@ -85,6 +85,7 @@ static void skel(const char *homedir, uid_t u, gid_t g) {
 			touch_file_as_user(fname, 0644);
 			fs_logger2("touch", fname);
 		}
+		selinux_relabel_path(fname, fname);
 		free(fname);
 	}
 	// bash etc.
@@ -105,6 +106,7 @@ static void skel(const char *homedir, uid_t u, gid_t g) {
 			fs_logger("clone /etc/skel/.bashrc");
 			fs_logger2("clone", fname);
 		}
+		selinux_relabel_path(fname, fname);
 		free(fname);
 	}
 }
@@ -139,6 +141,7 @@ static int store_xauthority(void) {
 
 		copy_file_as_user(src, dest, getuid(), getgid(), 0600); // regular user
 		fs_logger2("clone", dest);
+		selinux_relabel_path(dest, src);
 		free(src);
 		return 1; // file copied
 	}
@@ -185,6 +188,7 @@ static int store_asoundrc(void) {
 			errExit("fopen");
 
 		copy_file_as_user(src, dest, getuid(), getgid(), 0644); // regular user
+		selinux_relabel_path(dest, src);
 		fs_logger2("clone", dest);
 		free(src);
 		return 1; // file copied
@@ -208,6 +212,7 @@ static void copy_xauthority(void) {
 	}
 
 	copy_file_as_user(src, dest, getuid(), getgid(), S_IRUSR | S_IWUSR); // regular user
+	selinux_relabel_path(dest, src);
 	fs_logger2("clone", dest);
 	free(dest);
 
@@ -313,6 +318,7 @@ void fs_private_homedir(void) {
 			printf("Mounting a new /root directory\n");
 		if (mount("tmpfs", "/root", "tmpfs", MS_NOSUID | MS_NODEV | MS_NOEXEC | MS_STRICTATIME,  "mode=700,gid=0") < 0)
 			errExit("mounting /root directory");
+		selinux_relabel_path("/root", "/root");
 		fs_logger("tmpfs /root");
 	}
 	if (u == 0 && !arg_allusers) {
@@ -321,6 +327,7 @@ void fs_private_homedir(void) {
 			printf("Mounting a new /home directory\n");
 		if (mount("tmpfs", "/home", "tmpfs", MS_NOSUID | MS_NODEV | MS_NOEXEC | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 			errExit("mounting /home directory");
+		selinux_relabel_path("/home", "/home");
 		fs_logger("tmpfs /home");
 	}
 
@@ -345,45 +352,45 @@ void fs_private(void) {
 	int xflag = store_xauthority();
 	int aflag = store_asoundrc();
 
-	// mask /home
-	if (u == 0 && arg_allusers) // allow --allusers when starting the sandbox as root
-		;
-	else {
-		if (arg_debug)
-			printf("Mounting a new /home directory\n");
-		if (arg_allusers)
-			fwarning("allusers option disabled by private or whitelist option\n");
-		if (mount("tmpfs", "/home", "tmpfs", MS_NOSUID | MS_NODEV | MS_NOEXEC | MS_STRICTATIME,  "mode=755,gid=0") < 0)
-			errExit("mounting /home directory");
-		fs_logger("tmpfs /home");
-	}
-
 	// mask /root
 	if (arg_debug)
 		printf("Mounting a new /root directory\n");
 	if (mount("tmpfs", "/root", "tmpfs", MS_NOSUID | MS_NODEV | MS_NOEXEC | MS_STRICTATIME,  "mode=700,gid=0") < 0)
 		errExit("mounting /root directory");
+	selinux_relabel_path("/root", "/root");
 	fs_logger("tmpfs /root");
 
+	// mask /home
+	if (!arg_allusers) {
+		if (arg_debug)
+			printf("Mounting a new /home directory\n");
+		if (mount("tmpfs", "/home", "tmpfs", MS_NOSUID | MS_NODEV | MS_NOEXEC | MS_STRICTATIME,  "mode=755,gid=0") < 0)
+			errExit("mounting /home directory");
+		selinux_relabel_path("/home", "/home");
+		fs_logger("tmpfs /home");
+	}
+
 	if (u != 0) {
-		if (strncmp(homedir, "/home/", 6) == 0) {
-			// create /home/user
+		if (!arg_allusers && strncmp(homedir, "/home/", 6) == 0) {
+			// create new empty /home/user directory
 			if (arg_debug)
 				printf("Create a new user directory\n");
 			if (mkdir(homedir, S_IRWXU) == -1) {
 				if (mkpath_as_root(homedir) == -1)
 					errExit("mkpath");
-				if (mkdir(homedir, S_IRWXU) == -1 && errno != EEXIST)
+				if (mkdir(homedir, S_IRWXU) == -1)
 					errExit("mkdir");
 			}
 			if (chown(homedir, u, g) < 0)
 				errExit("chown");
+
+			selinux_relabel_path(homedir, homedir);
 			fs_logger2("mkdir", homedir);
 			fs_logger2("tmpfs", homedir);
 		}
 		else
-			// user directory is outside /home, mask it as well
-			// check if directory is owned by the current user
+			// mask user home directory
+			// the directory should be owned by the current user
 			fs_tmpfs(homedir, 1);
 	}
 
@@ -542,6 +549,7 @@ void fs_private_home_list(void) {
 
 	// create /run/firejail/mnt/home directory
 	mkdir_attr(RUN_HOME_DIR, 0755, uid, gid);
+	selinux_relabel_path(RUN_HOME_DIR, "/home");
 	fs_logger_print();	// save the current log
 
 	if (arg_debug)
@@ -604,6 +612,7 @@ void fs_private_home_list(void) {
 			printf("Mounting a new /root directory\n");
 		if (mount("tmpfs", "/root", "tmpfs", MS_NOSUID | MS_NODEV | MS_STRICTATIME,  "mode=700,gid=0") < 0)
 			errExit("mounting /root directory");
+		selinux_relabel_path("/root", "/root");
 		fs_logger("tmpfs /root");
 	}
 	if (uid == 0 && !arg_allusers) {
@@ -612,6 +621,7 @@ void fs_private_home_list(void) {
 			printf("Mounting a new /home directory\n");
 		if (mount("tmpfs", "/home", "tmpfs", MS_NOSUID | MS_NODEV | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 			errExit("mounting /home directory");
+		selinux_relabel_path("/home", "/home");
 		fs_logger("tmpfs /home");
 	}
 
