@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -346,6 +346,39 @@ static void whitelist_home(int topdir) {
 }
 
 
+static void globbing(const char *pattern) {
+	assert(pattern);
+
+	// globbing
+	glob_t globbuf;
+	int globerr = glob(pattern, GLOB_NOCHECK | GLOB_NOSORT | GLOB_PERIOD, NULL, &globbuf);
+	if (globerr) {
+		fprintf(stderr, "Error: failed to glob private-bin pattern %s\n", pattern);
+		exit(1);
+	}
+
+	size_t i;
+	for (i = 0; i < globbuf.gl_pathc; i++) {
+		assert(globbuf.gl_pathv[i]);
+		// testing for GLOB_NOCHECK - no pattern matched returns the original pattern
+		if (strcmp(globbuf.gl_pathv[i], pattern) == 0)
+			continue;
+
+		// build the new profile command
+		char *newcmd;
+		if (asprintf(&newcmd, "whitelist %s", globbuf.gl_pathv[i]) == -1)
+			errExit("asprintf");
+
+		// add the new profile command at the end of the list
+		if (arg_debug || arg_debug_whitelists)
+			printf("Adding new profile command: %s\n", newcmd);
+		profile_add(newcmd);
+	}
+
+	globfree(&globbuf);
+}
+
+
 void fs_whitelist(void) {
 	ProfileEntry *entry = cfg.profile;
 	if (!entry)
@@ -444,6 +477,13 @@ void fs_whitelist(void) {
 		else
 			fname = realpath(new_name, NULL);
 
+		// if this is not a real path, let's try globbing
+		// mark this entry as EMPTY_STRING and push the new paths at the end of profile entry list
+		// the new profile entries will be processed in this loop
+		// currently there is no globbing support for nowhitelist
+		if (!fname && !nowhitelist_flag)
+			globbing(new_name);
+
 		if (!fname) {
 			// file not found, blank the entry in the list and continue
 			if (arg_debug || arg_debug_whitelists) {
@@ -734,6 +774,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /tmp directory\n");
 			if (mount("tmpfs", "/tmp", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=1777,gid=0") < 0)
 				errExit("mounting tmpfs on /tmp");
+			selinux_relabel_path("/tmp", "/tmp");
 			fs_logger("tmpfs /tmp");
 
 			// pam-tmpdir - issue #2685
@@ -745,8 +786,10 @@ void fs_whitelist(void) {
 				if (strcmp(env, pamtmpdir) == 0) {
 					// create empty user-owned /tmp/user/$uid directory
 					mkdir_attr("/tmp/user", 0711, 0, 0);
+					selinux_relabel_path("/tmp/user", "/tmp/user");
 					fs_logger("mkdir /tmp/user");
 					mkdir_attr(pamtmpdir, 0700, getuid(), 0);
+					selinux_relabel_path(pamtmpdir, pamtmpdir);
 					fs_logger2("mkdir", pamtmpdir);
 				}
 				free(pamtmpdir);
@@ -774,6 +817,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /media directory\n");
 			if (mount("tmpfs", "/media", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /media");
+			selinux_relabel_path("/media", "/media");
 			fs_logger("tmpfs /media");
 
 			// autowhitelist home directory if it is masked by the tmpfs
@@ -798,6 +842,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /mnt directory\n");
 			if (mount("tmpfs", "/mnt", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /mnt");
+			selinux_relabel_path("/mnt", "/mnt");
 			fs_logger("tmpfs /mnt");
 
 			// autowhitelist home directory if it is masked by the tmpfs
@@ -822,6 +867,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /var directory\n");
 			if (mount("tmpfs", "/var", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /var");
+			selinux_relabel_path("/var", "/var");
 			fs_logger("tmpfs /var");
 
 			// autowhitelist home directory if it is masked by the tmpfs
@@ -846,6 +892,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /dev directory\n");
 			if (mount("tmpfs", "/dev", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /dev");
+			selinux_relabel_path("/dev", "/dev");
 			fs_logger("tmpfs /dev");
 
 			// autowhitelist home directory if it is masked by the tmpfs
@@ -870,6 +917,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /opt directory\n");
 			if (mount("tmpfs", "/opt", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /opt");
+			selinux_relabel_path("/opt", "/opt");
 			fs_logger("tmpfs /opt");
 
 			// autowhitelist home directory if it is masked by the tmpfs
@@ -894,6 +942,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /srv directory\n");
 			if (mount("tmpfs", "/srv", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /srv");
+			selinux_relabel_path("/srv", "/srv");
 			fs_logger("tmpfs /srv");
 
 			// autowhitelist home directory if it is masked by the tmpfs
@@ -918,6 +967,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /etc directory\n");
 			if (mount("tmpfs", "/etc", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /etc");
+			selinux_relabel_path("/etc", "/etc");
 			fs_logger("tmpfs /etc");
 
 			// autowhitelist home directory if it is masked by the tmpfs
@@ -942,6 +992,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /usr/share directory\n");
 			if (mount("tmpfs", "/usr/share", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /usr/share");
+			selinux_relabel_path("/usr/share", "/usr/share");
 			fs_logger("tmpfs /usr/share");
 
 			// autowhitelist home directory if it is masked by the tmpfs
@@ -966,6 +1017,7 @@ void fs_whitelist(void) {
 				printf("Mounting tmpfs on /sys/module directory\n");
 			if (mount("tmpfs", "/sys/module", "tmpfs", MS_NOSUID | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 				errExit("mounting tmpfs on /sys/module");
+			selinux_relabel_path("/sys/module", "/sys/module");
 			fs_logger("tmpfs /sys/module");
 		}
 		else
@@ -989,6 +1041,7 @@ void fs_whitelist(void) {
 				errExit("asprintf");
 			if (mount("tmpfs", runuser, "tmpfs", MS_NOSUID | MS_NODEV | MS_STRICTATIME, options) < 0)
 				errExit("mounting tmpfs on /run/user/<uid>");
+			selinux_relabel_path(runuser, runuser);
 			free(options);
 			fs_logger2("tmpfs", runuser);
 
@@ -1017,7 +1070,7 @@ void fs_whitelist(void) {
 			free(proc);
 			close(fd);
 
-			// mount a tmpfs and initialize home directory, overrides --allusers
+			// mount a tmpfs and initialize home directory
 			fs_private();
 		}
 		else
