@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -26,6 +26,7 @@
 #include <dirent.h>
 #include <pwd.h>
 #include <grp.h>
+#include <fcntl.h>
 //#include <dirent.h>
 //#include <stdio.h>
 //#include <stdlib.h>
@@ -34,18 +35,12 @@
 static uid_t c_uid = 0;
 static char *c_uid_name = NULL;
 
-static void print_file_or_dir(const char *path, const char *fname, int separator) {
+static void print_file_or_dir(const char *path, const char *fname) {
 	assert(fname);
 
 	char *name;
-	if (separator) {
-		if (asprintf(&name, "%s/%s", path, fname) == -1)
-			errExit("asprintf");
-	}
-	else {
-		if (asprintf(&name, "%s%s", path, fname) == -1)
-			errExit("asprintf");
-	}
+	if (asprintf(&name, "%s/%s", path, fname) == -1)
+		errExit("asprintf");
 
 	struct stat s;
 	if (stat(name, &s) == -1) {
@@ -54,6 +49,7 @@ static void print_file_or_dir(const char *path, const char *fname, int separator
 			return;
 		}
 	}
+	free(name);
 
 	// permissions
 	if (S_ISLNK(s.st_mode))
@@ -177,14 +173,83 @@ static void print_directory(const char *path) {
 	if (n < 0)
         		errExit("scandir");
 	else {
-		for (i = 0; i < n; i++) {
-			print_file_or_dir(path, namelist[i]->d_name, 0);
+		for (i = 0; i < n; i++)
+			print_file_or_dir(path, namelist[i]->d_name);
+		// get rid of false psitive reported by GCC -fanalyze
+		for (i = 0; i < n; i++)
 			free(namelist[i]);
-		}
 	}
 	free(namelist);
 }
 
+void ls(const char *path) {
+	EUID_ASSERT();
+	assert(path);
+
+	char *rp = realpath(path, NULL);
+	if (!rp || access(rp, R_OK) == -1) {
+		fprintf(stderr, "Error: cannot access %s\n", path);
+		exit(1);
+	}
+	if (arg_debug)
+		printf("ls %s\n", rp);
+
+	// list directory contents
+	struct stat s;
+	if (stat(rp, &s) == -1) {
+		fprintf(stderr, "Error: cannot access %s\n", rp);
+		exit(1);
+	}
+	if (S_ISDIR(s.st_mode))
+		print_directory(rp);
+	else {
+		char *split = strrchr(rp, '/');
+		if (split) {
+			*split = '\0';
+			char *rp2 = split + 1;
+			if (arg_debug)
+				printf("path %s, file %s\n", rp, rp2);
+			print_file_or_dir(rp, rp2);
+		}
+	}
+	free(rp);
+}
+
+void cat(const char *path) {
+	EUID_ASSERT();
+	assert(path);
+
+	if (arg_debug)
+		printf("cat %s\n", path);
+	FILE *fp = fopen(path, "r");
+	if (!fp) {
+		fprintf(stderr, "Error: cannot read %s\n", path);
+		exit(1);
+	}
+	int fd = fileno(fp);
+	if (fd == -1)
+		errExit("fileno");
+	struct stat s;
+	if (fstat(fd, &s) == -1)
+		errExit("fstat");
+	if (!S_ISREG(s.st_mode)) {
+		fprintf(stderr, "Error: %s is not a regular file\n", path);
+		exit(1);
+	}
+	bool tty = isatty(STDOUT_FILENO);
+
+	int c;
+	while ((c = fgetc(fp)) != EOF) {
+		// file is untrusted
+		// replace control characters when printing to a terminal
+		if (tty && c != '\t' && c != '\n' && iscntrl((unsigned char) c))
+			c = '?';
+		fputc(c, stdout);
+	}
+	fflush(stdout);
+	fclose(fp);
+}
+
 char *expand_path(const char *path) {
 	char *fname = NULL;
 	if (*path == '/') {
@@ -215,31 +280,53 @@ void sandboxfs(int op, pid_t pid, const char *path1, const char *path2) {
 	// in case the pid is that of a firejail process, use the pid of the first child process
 	pid = switch_to_child(pid);
 
-	// now check if the pid belongs to a firejail sandbox
-	if (invalid_sandbox(pid)) {
-		fprintf(stderr, "Error: no valid sandbox\n");
-		exit(1);
-	}
-
-	// check privileges for non-root users
-	uid_t uid = getuid();
-	if (uid != 0) {
-		uid_t sandbox_uid = pid_get_uid(pid);
-		if (uid != sandbox_uid) {
-			fprintf(stderr, "Error: permission denied.\n");
-			exit(1);
-		}
-	}
+	// exit if no permission to join the sandbox
+	check_join_permission(pid);
 
 	// expand paths
-	char *fname1 = expand_path(path1);;
+	char *fname1 = expand_path(path1);
 	char *fname2 = NULL;
 	if (path2 != NULL) {
 		fname2 = expand_path(path2);
 	}
 	if (arg_debug) {
 		printf("file1 %s\n", fname1);
-		printf("file2 %s\n", fname2);
+		printf("file2 %s\n", fname2 ? fname2 : "(null)");
+	}
+
+	// get file from sandbox and store it in the current directory
+	// implemented using --cat
+	if (op == SANDBOX_FS_GET) {
+		char *dest_fname = strrchr(fname1, '/');
+		if (!dest_fname || *(++dest_fname) == '\0') {
+			fprintf(stderr, "Error: invalid file name %s\n", fname1);
+			exit(1);
+		}
+		// create destination file if necessary
+		EUID_ASSERT();
+		int fd = open(dest_fname, O_WRONLY|O_CREAT|O_CLOEXEC, S_IRUSR | S_IWRITE);
+		if (fd == -1) {
+			fprintf(stderr, "Error: cannot open %s for writing\n", dest_fname);
+			exit(1);
+		}
+		struct stat s;
+		if (fstat(fd, &s) == -1)
+			errExit("fstat");
+		if (!S_ISREG(s.st_mode)) {
+			fprintf(stderr, "Error: %s is no regular file\n", dest_fname);
+			exit(1);
+		}
+		if (ftruncate(fd, 0) == -1)
+			errExit("ftruncate");
+		// go quiet - messages on stdout will corrupt the file
+		arg_debug = 0;
+		arg_quiet = 1;
+		// redirection
+		if (dup2(fd, STDOUT_FILENO) == -1)
+			errExit("dup2");
+		assert(fd != STDOUT_FILENO);
+		close(fd);
+		op = SANDBOX_FS_CAT;
 	}
 
 	// sandbox root directory
@@ -247,7 +334,7 @@ void sandboxfs(int op, pid_t pid, const char *path1, const char *path2) {
 	if (asprintf(&rootdir, "/proc/%d/root", pid) == -1)
 		errExit("asprintf");
 
-	if (op == SANDBOX_FS_LS) {
+	if (op == SANDBOX_FS_LS || op == SANDBOX_FS_CAT) {
 		EUID_ROOT();
 		// chroot
 		if (chroot(rootdir) < 0)
@@ -258,130 +345,13 @@ void sandboxfs(int op, pid_t pid, const char *path1, const char *path2) {
 		// drop privileges
 		drop_privs(0);
 
-		// check access
-		if (access(fname1, R_OK) == -1) {
-			fprintf(stderr, "Error: Cannot access %s\n", fname1);
-			exit(1);
-		}
-		/* coverity[toctou] */
-		char *rp = realpath(fname1, NULL);
-		if (!rp) {
-			fprintf(stderr, "Error: Cannot access %s\n", fname1);
-			exit(1);
-		}
-		if (arg_debug)
-			printf("realpath %s\n", rp);
-
-
-		// list directory contents
-		struct stat s;
-		if (stat(rp, &s) == -1) {
-			fprintf(stderr, "Error: Cannot access %s\n", rp);
-			exit(1);
-		}
-		if (S_ISDIR(s.st_mode)) {
-			char *dir;
-			if (asprintf(&dir, "%s/", rp) == -1)
-				errExit("asprintf");
-
-			print_directory(dir);
-			free(dir);
-		}
-		else {
-			char *split = strrchr(rp, '/');
-			if (split) {
-				*split = '\0';
-				char *rp2 = split + 1;
-				if (arg_debug)
-					printf("path %s, file %s\n", rp, rp2);
-				print_file_or_dir(rp, rp2, 1);
-			}
-		}
-		free(rp);
-	}
-
-	// get file from sandbox and store it in the current directory
-	else if (op == SANDBOX_FS_GET) {
-		char *src_fname =fname1;
-		char *dest_fname = strrchr(fname1, '/');
-		if (!dest_fname || *(++dest_fname) == '\0') {
-			fprintf(stderr, "Error: invalid file name %s\n", fname1);
-			exit(1);
-		}
-
-		EUID_ROOT();
-		if (arg_debug)
-			printf("copy %s to %s\n", src_fname, dest_fname);
-
-		// create a user-owned temporary file in /run/firejail directory
-		char tmp_fname[] = "/run/firejail/tmpget-XXXXXX";
-		int fd = mkstemp(tmp_fname);
-		if (fd != -1) {
-			SET_PERMS_FD(fd, getuid(), getgid(), 0600);
-			close(fd);
-		}
-
-		// copy the source file into the temporary file - we need to chroot
-		pid_t child = fork();
-		if (child < 0)
-			errExit("fork");
-		if (child == 0) {
-			// chroot
-			if (chroot(rootdir) < 0)
-				errExit("chroot");
-			if (chdir("/") < 0)
-				errExit("chdir");
-
-			// drop privileges
-			drop_privs(0);
-
-			// copy the file
-			if (copy_file(src_fname, tmp_fname, getuid(), getgid(), 0600)) // already a regular user
-				_exit(1);
+		if (op == SANDBOX_FS_LS)
+			ls(fname1);
+		else
+			cat(fname1);
 #ifdef HAVE_GCOV
-			__gcov_flush();
+		__gcov_flush();
 #endif
-			_exit(0);
-		}
-
-		// wait for the child to finish
-		int status = 0;
-		waitpid(child, &status, 0);
-		if (WIFEXITED(status) && WEXITSTATUS(status) == 0);
-		else {
-			unlink(tmp_fname);
-			exit(1);
-		}
-
-		// copy the temporary file into the destination file
-		child = fork();
-		if (child < 0)
-			errExit("fork");
-		if (child == 0) {
-			// drop privileges
-			drop_privs(0);
-
-			// copy the file
-			if (copy_file(tmp_fname, dest_fname, getuid(), getgid(), 0600)) // already a regular user
-				_exit(1);
-#ifdef HAVE_GCOV
-			__gcov_flush();
-#endif
-			_exit(0);
-		}
-
-		// wait for the child to finish
-		status = 0;
-		waitpid(child, &status, 0);
-		if (WIFEXITED(status) && WEXITSTATUS(status) == 0);
-		else {
-			unlink(tmp_fname);
-			exit(1);
-		}
-
-		// remove the temporary file
-		unlink(tmp_fname);
-		EUID_USER();
 	}
 	// get file from host and store it in the sandbox
 	else if (op == SANDBOX_FS_PUT && path2) {
