@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -208,7 +208,11 @@ char *expand_macros(const char *path) {
 
 	// Replace home macro
 	char *new_name = NULL;
-	if (strncmp(path, "${HOME}", 7) == 0) {
+	if (strncmp(path, "$HOME", 5) == 0) {
+		fprintf(stderr, "Error: $HOME is not allowed in profile files, please replace it with ${HOME}\n");
+		exit(1);
+	}
+	else if (strncmp(path, "${HOME}", 7) == 0) {
 		if (asprintf(&new_name, "%s%s", cfg.homedir, path + 7) == -1)
 			errExit("asprintf");
 		if(called_as_root)
@@ -300,9 +304,8 @@ void invalid_filename(const char *fname, int globbing) {
 	size_t i = 0;
 	while (ptr[i] != '\0') {
 		if (iscntrl((unsigned char) ptr[i])) {
-			char *msg = fix_control_chars(fname);
-			fprintf(stderr, "Error: \"%s\" is an invalid filename: no control characters allowed\n", msg);
-			free(msg);
+			char *new = fix_control_chars(fname);
+			fprintf(stderr, "Error: \"%s\" is an invalid filename: no control characters allowed\n", new);
 			exit(1);
 		}
 		i++;
