@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -20,6 +20,8 @@
 #include "firejail.h"
 #include "../include/pid.h"
 #include "../include/firejail_user.h"
+#include "../include/syscall.h"
+#include "../include/seccomp.h"
 #define _GNU_SOURCE
 #include <sys/utsname.h>
 #include <sched.h>
@@ -29,7 +31,7 @@
 #include <dirent.h>
 #include <pwd.h>
 #include <errno.h>
-#include <limits.h>
+//#include <limits.h>
 #include <sys/file.h>
 #include <sys/prctl.h>
 #include <signal.h>
@@ -72,8 +74,10 @@ int arg_overlay_keep = 0;			// place overlay diff in a known directory
 int arg_overlay_reuse = 0;			// allow the reuse of overlays
 
 int arg_seccomp = 0;				// enable default seccomp filter
+int arg_seccomp32 = 0;				// enable default seccomp filter for 32 bit arch
 int arg_seccomp_postexec = 0;			// need postexec ld.preload library?
 int arg_seccomp_block_secondary = 0;		// block any secondary architectures
+int arg_seccomp_error_action = 0;
 
 int arg_caps_default_filter = 0;			// enable default capabilities filter
 int arg_caps_drop = 0;				// drop list
@@ -140,12 +144,15 @@ int arg_noprofile = 0; // use default.profile if none other found/specified
 int arg_memory_deny_write_execute = 0;		// block writable and executable memory
 int arg_notv = 0;	// --notv
 int arg_nodvd = 0; // --nodvd
-int arg_nodbus = 0; // -nodbus
 int arg_nou2f = 0; // --nou2f
 int arg_deterministic_exit_code = 0;	// always exit with first child's exit status
+DbusPolicy arg_dbus_user = DBUS_POLICY_ALLOW;	// --dbus-user
+DbusPolicy arg_dbus_system = DBUS_POLICY_ALLOW;	// --dbus-system
+const char *arg_dbus_log_file = NULL;
+int arg_dbus_log_user = 0;
+int arg_dbus_log_system = 0;
 int login_shell = 0;
 
-
 int parent_to_child_fds[2];
 int child_to_parent_fds[2];
 
@@ -154,7 +161,6 @@ int fullargc = 0;
 static pid_t child = 0;
 pid_t sandbox_pid;
 mode_t orig_umask = 022;
-unsigned long long start_timestamp;
 
 static void clear_atexit(void) {
 	EUID_ROOT();
@@ -168,6 +174,9 @@ static void myexit(int rv) {
 
 
 	// delete sandbox files in shared memory
+#ifdef HAVE_DBUSPROXY
+	dbus_proxy_stop();
+#endif
 	EUID_ROOT();
 	delete_run_files(sandbox_pid);
 	appimage_clear();
@@ -178,6 +187,7 @@ static void myexit(int rv) {
 static void my_handler(int s) {
 	fmessage("\nParent received signal %d, shutting down the child process...\n", s);
 	logsignal(s);
+
 	if (waitpid(child, NULL, WNOHANG) == 0) {
 		if (has_handler(child, s)) // signals are not delivered if there is no handler yet
 			kill(child, s);
@@ -206,74 +216,6 @@ static void install_handler(void) {
 	sigaction(SIGTERM, &sga, NULL);
 }
 
-// return 1 if error, 0 if a valid pid was found
-static int extract_pid(const char *name, pid_t *pid) {
-	int retval = 0;
-	EUID_ASSERT();
-	if (!name || strlen(name) == 0) {
-		fprintf(stderr, "Error: invalid sandbox name\n");
-		exit(1);
-	}
-
-	EUID_ROOT();
-	if (name2pid(name, pid)) {
-		retval = 1;
-	}
-	EUID_USER();
-	return retval;
-}
-
-// return 1 if error, 0 if a valid pid was found
-static int read_pid(const char *name, pid_t *pid) {
-	char *endptr;
-	errno = 0;
-	long int pidtmp = strtol(name, &endptr, 10);
-	if ((errno == ERANGE && (pidtmp == LONG_MAX || pidtmp == LONG_MIN))
-		|| (errno != 0 && pidtmp == 0)) {
-		return extract_pid(name,pid);
-	}
-	// endptr points to '\0' char in name if the entire string is valid
-	if (endptr == NULL || endptr[0]!='\0') {
-		return extract_pid(name,pid);
-	}
-	*pid =(pid_t)pidtmp;
-	return 0;
-}
-
-static pid_t require_pid(const char *name) {
-	pid_t pid;
-	if (read_pid(name,&pid)) {
-		fprintf(stderr, "Error: cannot find sandbox %s\n", name);
-		exit(1);
-	}
-	return pid;
-}
-
-// return 1 if there is a link somewhere in path of directory
-static int has_link(const char *dir) {
-	assert(dir);
-	int fd = safe_fd(dir, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
-	if (fd == -1) {
-		if (errno == ENOTDIR && is_dir(dir))
-			return 1;
-	}
-	else
-		close(fd);
-	return 0;
-}
-
-static void check_homedir(void) {
-	assert(cfg.homedir);
-	if (cfg.homedir[0] != '/') {
-		fprintf(stderr, "Error: invalid user directory \"%s\"\n", cfg.homedir);
-		exit(1);
-	}
-	// symlinks are rejected in many places
-	if (has_link(cfg.homedir)) {
-		fprintf(stderr, "No full support for symbolic links in path of user directory.\n"
-			"Please provide resolved path in password database (/etc/passwd).\n\n");
-	}
-}
 
 // init configuration
 static void init_cfg(int argc, char **argv) {
@@ -325,6 +267,27 @@ static void init_cfg(int argc, char **argv) {
 	sandbox_pid = getpid();
 	time_t t = time(NULL);
 	srand(t ^ sandbox_pid);
+
+	arg_seccomp_error_action = EPERM;
+	cfg.seccomp_error_action = "EPERM";
+}
+
+static void fix_single_std_fd(int fd, const char *file, int flags) {
+	struct stat s;
+	if (fstat(fd, &s) == -1 && errno == EBADF) {
+		// something is wrong with fd, probably it is not opened
+		int nfd = open(file, flags);
+		if (nfd != fd || fstat(fd, &s) != 0)
+			_exit(1); // no further attempts to fix the situation
+	}
+}
+
+// glibc does this automatically if Firejail was started by a regular user
+// run this for root user and as a fallback
+static void fix_std_streams(void) {
+	fix_single_std_fd(0, "/dev/full", O_RDONLY|O_NOFOLLOW);
+	fix_single_std_fd(1, "/dev/null", O_WRONLY|O_NOFOLLOW);
+	fix_single_std_fd(2, "/dev/null", O_WRONLY|O_NOFOLLOW);
 }
 
 static void check_network(Bridge *br) {
@@ -517,7 +480,6 @@ static void run_cmd_and_exit(int i, int argc, char **argv) {
 	//*************************************
 	// independent commands - the program will exit!
 	//*************************************
-#ifdef HAVE_SECCOMP
 	else if (strcmp(argv[i], "--debug-syscalls") == 0) {
 		if (checkcfg(CFG_SECCOMP)) {
 			int rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2, PATH_FSECCOMP_MAIN, "debug-syscalls");
@@ -526,6 +488,14 @@ static void run_cmd_and_exit(int i, int argc, char **argv) {
 		else
 			exit_err_feature("seccomp");
 	}
+	else if (strcmp(argv[i], "--debug-syscalls32") == 0) {
+		if (checkcfg(CFG_SECCOMP)) {
+			int rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2, PATH_FSECCOMP_MAIN, "debug-syscalls32");
+			exit(rv);
+		}
+		else
+			exit_err_feature("seccomp");
+	}
 	else if (strcmp(argv[i], "--debug-errnos") == 0) {
 		if (checkcfg(CFG_SECCOMP)) {
 			int rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2, PATH_FSECCOMP_MAIN, "debug-errnos");
@@ -559,7 +529,6 @@ static void run_cmd_and_exit(int i, int argc, char **argv) {
 			exit_err_feature("seccomp");
 		exit(0);
 	}
-#endif
 	else if (strncmp(argv[i], "--profile.print=", 16) == 0) {
 		pid_t pid = require_pid(argv[i] + 16);
 
@@ -751,6 +720,8 @@ static void run_cmd_and_exit(int i, int argc, char **argv) {
 			 }
 
 			// list directory contents
+			if (!arg_debug)
+				 arg_quiet = 1;
 			pid_t pid = require_pid(argv[i] + 5);
 			sandboxfs(SANDBOX_FS_LS, pid, path, NULL);
 			exit(0);
@@ -758,6 +729,35 @@ static void run_cmd_and_exit(int i, int argc, char **argv) {
 		else
 			exit_err_feature("file transfer");
 	}
+	else if (strncmp(argv[i], "--cat=", 6) == 0) {
+		if (checkcfg(CFG_FILE_TRANSFER)) {
+			logargs(argc, argv);
+			if (arg_private_cwd) {
+				fprintf(stderr, "Error: --cat and --private-cwd options are mutually exclusive\n");
+				exit(1);
+			}
+
+			if ((i + 2) != argc) {
+				fprintf(stderr, "Error: invalid --cat option, path expected\n");
+				exit(1);
+			}
+			char *path = argv[i + 1];
+			invalid_filename(path, 0); // no globbing
+			if (strstr(path, "..")) {
+				fprintf(stderr, "Error: invalid file name %s\n", path);
+				exit(1);
+			}
+
+			// write file contents to stdout
+			if (!arg_debug)
+				 arg_quiet = 1;
+			pid_t pid = require_pid(argv[i] + 6);
+			sandboxfs(SANDBOX_FS_CAT, pid, path, NULL);
+			exit(0);
+		}
+		else
+			exit_err_feature("file transfer");
+	}
 #endif
 	else if (strncmp(argv[i], "--join=", 7) == 0) {
 		if (checkcfg(CFG_JOIN) || getuid() == 0) {
@@ -867,7 +867,8 @@ char *guess_shell(void) {
 	shell = getenv("SHELL");
 	if (shell) {
 		invalid_filename(shell, 0); // no globbing
-		if (!is_dir(shell) && strstr(shell, "..") == NULL && stat(shell, &s) == 0 && access(shell, X_OK) == 0)
+		if (!is_dir(shell) && strstr(shell, "..") == NULL && stat(shell, &s) == 0 && access(shell, X_OK) == 0 &&
+		    strcmp(shell, PATH_FIREJAIL) != 0)
 			return shell;
 	}
 
@@ -887,19 +888,20 @@ char *guess_shell(void) {
 	return shell;
 }
 
+// return argument index
 static int check_arg(int argc, char **argv, const char *argument, int strict) {
 	int i;
 	int found = 0;
 	for (i = 1; i < argc; i++) {
 		if (strict) {
 			if (strcmp(argv[i], argument) == 0) {
-				found = 1;
+				found = i;
 				break;
 			}
 		}
 		else {
 			if (strncmp(argv[i], argument, strlen(argument)) == 0) {
-				found = 1;
+				found = i;
 				break;
 			}
 		}
@@ -934,11 +936,36 @@ static void run_builder(int argc, char **argv) {
 	exit(1);
 }
 
+void filter_add_errno(int fd, int syscall, int arg, void *ptrarg, bool native) {
+	(void) fd;
+	(void) syscall;
+	(void) arg;
+	(void) ptrarg;
+	(void) native;
+}
+void filter_add_blacklist_override(int fd, int syscall, int arg, void *ptrarg, bool native) {
+	(void) fd;
+	(void) syscall;
+	(void) arg;
+	(void) ptrarg;
+	(void) native;
+}
+
+static int check_postexec(const char *list) {
+	char *prelist, *postlist;
+
+	if (list) {
+		syscalls_in_list(list, "@default-keep", -1, &prelist, &postlist, true);
+		if (postlist)
+			return 1;
+	}
+	return 0;
+}
 
 //*******************************************
 // Main program
 //*******************************************
-int main(int argc, char **argv) {
+int main(int argc, char **argv, char **envp) {
 	int i;
 	int prog_index = -1;			  // index in argv where the program command starts
 	int lockfd_network = -1;
@@ -946,17 +973,52 @@ int main(int argc, char **argv) {
 	int option_cgroup = 0;
 	int custom_profile = 0;	// custom profile loaded
 	int arg_caps_cmdline = 0; 	// caps requested on command line (used to break out of --chroot)
+	char **ptr;
+
+	// sanitize the umask
+	orig_umask = umask(022);
+
+	// check standard streams before printing anything
+	fix_std_streams();
 
 	// drop permissions by default and rise them when required
 	EUID_INIT();
 	EUID_USER();
 
-	// sanitize the umask
-	orig_umask = umask(022);
-
 	// argument count should be larger than 0
-	if (argc == 0) {
-		fprintf(stderr, "Error: argv[0] is NULL\n");
+	if (argc == 0 || !argv || strlen(argv[0]) == 0) {
+		fprintf(stderr, "Error: argv is invalid\n");
+		exit(1);
+	} else if (argc >= MAX_ARGS) {
+		fprintf(stderr, "Error: too many arguments\n");
+		exit(1);
+	}
+
+	// sanity check for arguments
+	for (i = 0; i < argc; i++) {
+		if (*argv[i] == 0) {
+			fprintf(stderr, "Error: too short arguments\n");
+			exit(1);
+		}
+		if (strlen(argv[i]) >= MAX_ARG_LEN) {
+			fprintf(stderr, "Error: too long arguments\n");
+			exit(1);
+		}
+		// Also remove requested environment variables
+		// entirely to avoid tripping the length check below
+		if (strncmp(argv[i], "--rmenv=", 8) == 0)
+			unsetenv(argv[i] + 8);
+	}
+
+	// sanity check for environment variables
+	for (i = 0, ptr = envp; ptr && *ptr && i < MAX_ENVS; i++, ptr++) {
+		if (strlen(*ptr) >= MAX_ENV_LEN) {
+			fprintf(stderr, "Error: too long environment variables, please use --rmenv\n");
+			exit(1);
+		}
+	}
+	if (i >= MAX_ENVS) {
+		fprintf(stderr, "Error: too many environment variables, please use --rmenv\n");
 		exit(1);
 	}
 
@@ -964,7 +1026,7 @@ int main(int argc, char **argv) {
 	init_cfg(argc, argv);
 
 	// get starting timestamp, process --quiet
-	start_timestamp = getticks();
+	timetrace_start();
 	char *env_quiet = getenv("FIREJAIL_QUIET");
 	if (check_arg(argc, argv, "--quiet", 1) || (env_quiet && strcmp(env_quiet, "yes") == 0))
 		arg_quiet = 1;
@@ -989,6 +1051,19 @@ int main(int argc, char **argv) {
 	}
 	EUID_USER();
 
+	// --ip=dhcp - we need access to /sbin and /usr/sbin directories in order to run ISC DHCP client (dhclient)
+	// these paths are disabled in disable-common.inc
+	if ((i = check_arg(argc, argv, "--ip", 0)) != 0) {
+		if (strncmp(argv[i] + 4, "=dhcp", 5) == 0) {
+			profile_add("noblacklist /sbin");
+			profile_add("noblacklist /usr/sbin");
+		}
+	}
+
+	// for appimages we need to remove "include disable-shell.inc from the profile
+	// a --profile command can show up before --appimage
+	if (check_arg(argc, argv, "--appimage", 1))
+		arg_appimage = 1;
 
 	// process allow-debuggers
 	if (check_arg(argc, argv, "--allow-debuggers", 1)) {
@@ -1041,8 +1116,7 @@ int main(int argc, char **argv) {
 
 			// start the program directly without sandboxing
 			run_no_sandbox(argc, argv);
-			// it will never get here!
-			assert(0);
+			__builtin_unreachable();
 		}
 	}
 	EUID_ASSERT();
@@ -1157,7 +1231,6 @@ int main(int argc, char **argv) {
 	}
 	EUID_ASSERT();
 
-
 	// check for force-nonewprivs in /etc/firejail/firejail.config file
 	if (checkcfg(CFG_FORCE_NONEWPRIVS))
 		arg_nonewprivs = 1;
@@ -1202,11 +1275,10 @@ int main(int argc, char **argv) {
 		else if (strcmp(argv[i], "--apparmor") == 0)
 			arg_apparmor = 1;
 #endif
-#ifdef HAVE_SECCOMP
 		else if (strncmp(argv[i], "--protocol=", 11) == 0) {
 			if (checkcfg(CFG_SECCOMP)) {
 				if (cfg.protocol) {
-					fwarning("two protocol lists are present, \"%s\" will be installed\n", cfg.protocol);
+					fwarning("more than one protocol list is present, \"%s\" will be installed\n", cfg.protocol);
 				}
 				else {
 					// store list
@@ -1241,6 +1313,18 @@ int main(int argc, char **argv) {
 			else
 				exit_err_feature("seccomp");
 		}
+		else if (strncmp(argv[i], "--seccomp.32=", 13) == 0) {
+			if (checkcfg(CFG_SECCOMP)) {
+				if (arg_seccomp32) {
+					fprintf(stderr, "Error: seccomp.32 already enabled\n");
+					exit(1);
+				}
+				arg_seccomp32 = 1;
+				cfg.seccomp_list32 = seccomp_check_list(argv[i] + 13);
+			}
+			else
+				exit_err_feature("seccomp");
+		}
 		else if (strncmp(argv[i], "--seccomp.drop=", 15) == 0) {
 			if (checkcfg(CFG_SECCOMP)) {
 				if (arg_seccomp) {
@@ -1253,6 +1337,18 @@ int main(int argc, char **argv) {
 			else
 				exit_err_feature("seccomp");
 		}
+		else if (strncmp(argv[i], "--seccomp.32.drop=", 18) == 0) {
+			if (checkcfg(CFG_SECCOMP)) {
+				if (arg_seccomp32) {
+					fprintf(stderr, "Error: seccomp.32 already enabled\n");
+					exit(1);
+				}
+				arg_seccomp32 = 1;
+				cfg.seccomp_list_drop32 = seccomp_check_list(argv[i] + 18);
+			}
+			else
+				exit_err_feature("seccomp");
+		}
 		else if (strncmp(argv[i], "--seccomp.keep=", 15) == 0) {
 			if (checkcfg(CFG_SECCOMP)) {
 				if (arg_seccomp) {
@@ -1265,8 +1361,24 @@ int main(int argc, char **argv) {
 			else
 				exit_err_feature("seccomp");
 		}
+		else if (strncmp(argv[i], "--seccomp.32.keep=", 18) == 0) {
+			if (checkcfg(CFG_SECCOMP)) {
+				if (arg_seccomp32) {
+					fprintf(stderr, "Error: seccomp.32 already enabled\n");
+					exit(1);
+				}
+				arg_seccomp32 = 1;
+				cfg.seccomp_list_keep32 = seccomp_check_list(argv[i] + 18);
+			}
+			else
+				exit_err_feature("seccomp");
+		}
 		else if (strcmp(argv[i], "--seccomp.block-secondary") == 0) {
 			if (checkcfg(CFG_SECCOMP)) {
+				if (arg_seccomp32) {
+					fprintf(stderr, "Error: seccomp.32 conflicts with block-secondary\n");
+					exit(1);
+				}
 				arg_seccomp_block_secondary = 1;
 			}
 			else
@@ -1278,7 +1390,28 @@ int main(int argc, char **argv) {
 			else
 				exit_err_feature("seccomp");
 		}
-#endif
+		else if (strncmp(argv[i], "--seccomp-error-action=", 23) == 0) {
+			if (checkcfg(CFG_SECCOMP)) {
+				int config_seccomp_error_action = checkcfg(CFG_SECCOMP_ERROR_ACTION);
+				if (config_seccomp_error_action == -1) {
+					if (strcmp(argv[i] + 23, "kill") == 0)
+						arg_seccomp_error_action = SECCOMP_RET_KILL;
+					else if (strcmp(argv[i] + 23, "log") == 0)
+						arg_seccomp_error_action = SECCOMP_RET_LOG;
+					else {
+						arg_seccomp_error_action = errno_find_name(argv[i] + 23);
+						if (arg_seccomp_error_action == -1)
+							errExit("seccomp-error-action: unknown errno");
+					}
+					cfg.seccomp_error_action = strdup(argv[i] + 23);
+					if (!cfg.seccomp_error_action)
+						errExit("strdup");
+				} else
+					exit_err_feature("seccomp-error-action");
+
+			} else
+				exit_err_feature("seccomp");
+		}
 		else if (strcmp(argv[i], "--caps") == 0) {
 			arg_caps_default_filter = 1;
 			arg_caps_cmdline = 1;
@@ -1589,6 +1722,34 @@ int main(int argc, char **argv) {
 			}
 		}
 #endif
+		else if (strncmp(argv[i], "--include=", 10) == 0) {
+			char *ppath = expand_macros(argv[i] + 10);
+			if (!ppath)
+				errExit("strdup");
+
+			char *ptr = ppath;
+			while (*ptr != '/' && *ptr != '\0')
+				ptr++;
+			if (*ptr == '\0') {
+				if (access(ppath, R_OK)) {
+					profile_read(ppath);
+				}
+				else {
+					// ppath contains no '/' and is not a local file, assume it's a name
+					int rv = profile_find_firejail(ppath, 0);
+					if (!rv) {
+						fprintf(stderr, "Error: no profile with name \"%s\" found.\n", ppath);
+						exit(1);
+					}
+				}
+			}
+			else {
+				// ppath contains a '/', assume it's a path
+				profile_read(ppath);
+			}
+
+			free(ppath);
+		}
 		else if (strncmp(argv[i], "--profile=", 10) == 0) {
 			// multiple profile files are allowed!
 
@@ -1834,12 +1995,14 @@ int main(int argc, char **argv) {
 		else if (strcmp(argv[i], "--private-tmp") == 0) {
 			arg_private_tmp = 1;
 		}
+#ifdef HAVE_USERTMPFS
 		else if (strcmp(argv[i], "--private-cache") == 0) {
 			if (checkcfg(CFG_PRIVATE_CACHE))
 				arg_private_cache = 1;
 			else
 				exit_err_feature("private-cache");
 		}
+#endif
 		else if (strcmp(argv[i], "--private-cwd") == 0) {
 			cfg.cwd = NULL;
 			arg_private_cwd = 1;
@@ -1901,8 +2064,153 @@ int main(int argc, char **argv) {
 			arg_nodvd = 1;
 		else if (strcmp(argv[i], "--nou2f") == 0)
 			arg_nou2f = 1;
-		else if (strcmp(argv[i], "--nodbus") == 0)
-			arg_nodbus = 1;
+		else if (strcmp(argv[i], "--nodbus") == 0) {
+			arg_dbus_user = DBUS_POLICY_BLOCK;
+			arg_dbus_system = DBUS_POLICY_BLOCK;
+		}
+
+		//*************************************
+		// D-BUS proxy
+		//*************************************
+#ifdef HAVE_DBUSPROXY
+		else if (strncmp("--dbus-user=", argv[i], 12) == 0) {
+			if (strcmp("filter", argv[i] + 12) == 0) {
+				if (arg_dbus_user == DBUS_POLICY_BLOCK) {
+					fprintf(stderr, "Warning: Cannot relax --dbus-user policy, it is already set to block\n");
+				} else {
+					arg_dbus_user = DBUS_POLICY_FILTER;
+				}
+			} else if (strcmp("none", argv[i] + 12) == 0) {
+				if (arg_dbus_log_user) {
+					fprintf(stderr, "Error: --dbus-user.log requires --dbus-user=filter\n");
+					exit(1);
+				}
+				arg_dbus_user = DBUS_POLICY_BLOCK;
+			} else {
+				fprintf(stderr, "Unknown dbus-user policy: %s\n", argv[i] + 12);
+				exit(1);
+			}
+		}
+		else if (strncmp(argv[i], "--dbus-user.see=", 16) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-user.see %s", argv[i] + 16) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-user.talk=", 17) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-user.talk %s", argv[i] + 17) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-user.own=", 16) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-user.own %s", argv[i] + 16) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-user.call=", 17) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-user.call %s", argv[i] + 17) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-user.broadcast=", 22) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-user.broadcast %s", argv[i] + 22) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp("--dbus-system=", argv[i], 14) == 0) {
+			if (strcmp("filter", argv[i] + 14) == 0) {
+				if (arg_dbus_system == DBUS_POLICY_BLOCK) {
+					fprintf(stderr, "Warning: Cannot relax --dbus-system policy, it is already set to block\n");
+				} else {
+					arg_dbus_system = DBUS_POLICY_FILTER;
+				}
+			} else if (strcmp("none", argv[i] + 14) == 0) {
+				if (arg_dbus_log_system) {
+					fprintf(stderr, "Error: --dbus-system.log requires --dbus-system=filter\n");
+					exit(1);
+				}
+				arg_dbus_system = DBUS_POLICY_BLOCK;
+			} else {
+				fprintf(stderr, "Unknown dbus-system policy: %s\n", argv[i] + 14);
+				exit(1);
+			}
+		}
+		else if (strncmp(argv[i], "--dbus-system.see=", 18) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-system.see %s", argv[i] + 18) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-system.talk=", 19) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-system.talk %s", argv[i] + 19) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-system.own=", 18) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-system.own %s", argv[i] + 18) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-system.call=", 19) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-system.call %s", argv[i] + 19) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-system.broadcast=", 24) == 0) {
+			char *line;
+			if (asprintf(&line, "dbus-system.broadcast %s", argv[i] + 24) == -1)
+				errExit("asprintf");
+
+			profile_check_line(line, 0, NULL); // will exit if something wrong
+			profile_add(line);
+		}
+		else if (strncmp(argv[i], "--dbus-log=", 11) == 0) {
+			if (arg_dbus_log_file != NULL) {
+				fprintf(stderr, "Error: --dbus-log option already specified\n");
+				exit(1);
+			}
+			arg_dbus_log_file = argv[i] + 11;
+		}
+		else if (strcmp(argv[i], "--dbus-user.log") == 0) {
+			if (arg_dbus_user != DBUS_POLICY_FILTER) {
+				fprintf(stderr, "Error: --dbus-user.log requires --dbus-user=filter\n");
+				exit(1);
+			}
+			arg_dbus_log_user = 1;
+		}
+		else if (strcmp(argv[i], "--dbus-system.log") == 0) {
+			if (arg_dbus_system != DBUS_POLICY_FILTER) {
+				fprintf(stderr, "Error: --dbus-system.log requires --dbus-system=filter\n");
+				exit(1);
+			}
+			arg_dbus_log_system = 1;
+		}
+#endif
 
 		//*************************************
 		// network
@@ -2085,6 +2393,13 @@ int main(int argc, char **argv) {
 					fprintf(stderr, "Error: invalid MAC address\n");
 					exit(1);
 				}
+
+				// check multicast address
+				if (br->macsandbox[0] & 1) {
+					fprintf(stderr, "Error: invalid MAC address (multicast)\n");
+					exit(1);
+				}
+
 			}
 			else
 				exit_err_feature("networking");
@@ -2122,7 +2437,10 @@ int main(int argc, char **argv) {
 				// configure this IP address for the last bridge defined
 				if (strcmp(argv[i] + 5, "none") == 0)
 					br->arg_ip_none = 1;
-				else {
+				else if (strcmp(argv[i] + 5, "dhcp") == 0) {
+					br->arg_ip_none = 1;
+					br->arg_ip_dhcp = 1;
+				} else {
 					if (atoip(argv[i] + 5, &br->ipsandbox)) {
 						fprintf(stderr, "Error: invalid IP address\n");
 						exit(1);
@@ -2162,20 +2480,24 @@ int main(int argc, char **argv) {
 					fprintf(stderr, "Error: no network device configured\n");
 					exit(1);
 				}
-				if (br->ip6sandbox) {
+				if (br->arg_ip6_dhcp || br->ip6sandbox) {
 					fprintf(stderr, "Error: cannot configure the IP address twice for the same interface\n");
 					exit(1);
 				}
 
 				// configure this IP address for the last bridge defined
-				if (check_ip46_address(argv[i] + 6) == 0) {
-					fprintf(stderr, "Error: invalid IPv6 address\n");
-					exit(1);
-				}
+				if (strcmp(argv[i] + 6, "dhcp") == 0)
+					br->arg_ip6_dhcp = 1;
+				else {
+					if (check_ip46_address(argv[i] + 6) == 0) {
+						fprintf(stderr, "Error: invalid IPv6 address\n");
+						exit(1);
+					}
 
-				br->ip6sandbox = strdup(argv[i] + 6);
-				if (br->ip6sandbox == NULL)
-					errExit("strdup");
+					br->ip6sandbox = strdup(argv[i] + 6);
+					if (br->ip6sandbox == NULL)
+						errExit("strdup");
+				}
 			}
 			else
 				exit_err_feature("networking");
@@ -2264,6 +2586,7 @@ int main(int argc, char **argv) {
 			cfg.timeout = extract_timeout(argv[i] + 10);
 		else if (strcmp(argv[i], "--audit") == 0) {
 			arg_audit_prog = LIBDIR "/firejail/faudit";
+			profile_add_ignore("shell none");
 			arg_audit = 1;
 		}
 		else if (strncmp(argv[i], "--audit=", 8) == 0) {
@@ -2280,6 +2603,7 @@ int main(int argc, char **argv) {
 				fprintf(stderr, "Error: cannot find the audit program %s\n", arg_audit_prog);
 				exit(1);
 			}
+			profile_add_ignore("shell none");
 			arg_audit = 1;
 		}
 		else if (strcmp(argv[i], "--appimage") == 0)
@@ -2458,7 +2782,7 @@ int main(int argc, char **argv) {
 
 	// build the sandbox command
 	if (prog_index == -1 && cfg.shell) {
-		cfg.command_line = cfg.shell;
+		assert(cfg.command_line == NULL); // runs cfg.shell
 		cfg.window_title = cfg.shell;
 		cfg.command_name = cfg.shell;
 	}
@@ -2513,6 +2837,13 @@ int main(int argc, char **argv) {
 	// check network configuration options - it will exit if anything went wrong
 	net_check_cfg();
 
+	if (arg_seccomp)
+		arg_seccomp_postexec = check_postexec(cfg.seccomp_list) || check_postexec(cfg.seccomp_list_drop);
+
+	bool need_preload = arg_trace || arg_tracelog || arg_seccomp_postexec;
+	if (need_preload && (cfg.seccomp_list32 || cfg.seccomp_list_drop32 || cfg.seccomp_list_keep32))
+		fwarning("preload libraries (trace, tracelog, postexecseccomp due to seccomp.drop=execve etc.) are incompatible with 32 bit filters\n");
+
 	// check and assign an IP address - for macvlan it will be done again in the sandbox!
 	if (any_bridge_configured()) {
 		EUID_ROOT();
@@ -2573,6 +2904,18 @@ int main(int argc, char **argv) {
 	}
 	EUID_USER();
 
+#ifdef HAVE_DBUSPROXY
+	if (checkcfg(CFG_DBUS)) {
+		dbus_check_profile();
+		if (arg_dbus_user == DBUS_POLICY_FILTER ||
+			arg_dbus_system == DBUS_POLICY_FILTER) {
+			EUID_ROOT();
+			dbus_proxy_start();
+			EUID_USER();
+		}
+	}
+#endif
+
 	// clone environment
 	int flags = CLONE_NEWNS | CLONE_NEWPID | CLONE_NEWUTS | SIGCHLD;
 
@@ -2682,8 +3025,15 @@ int main(int argc, char **argv) {
 	 	ptr += strlen(ptr);
 
 	 	if (!arg_nogroups) {
+		 	//  add firejail group
+		 	gid_t g = get_group_id("firejail");
+		 	if (g) {
+		 		sprintf(ptr, "%d %d 1\n", g, g);
+		 		ptr += strlen(ptr);
+		 	}
+
 		 	//  add tty group
-		 	gid_t g = get_group_id("tty");
+		 	g = get_group_id("tty");
 		 	if (g) {
 		 		sprintf(ptr, "%d %d 1\n", g, g);
 		 		ptr += strlen(ptr);
@@ -2745,17 +3095,27 @@ int main(int argc, char **argv) {
 	// end of signal-safe code
 	//*****************************
 
+#if 0
+// at this point the sandbox was closed and we are on our way out
+// it would make sense to move this before waitpid above to free some memory
+// crash for now as of issue #3662 from dhcp code
 	// free globals
 	if (cfg.profile) {
 		ProfileEntry *prf = cfg.profile;
 		while (prf != NULL) {
 			ProfileEntry *next = prf->next;
-			free(prf->data);
-			free(prf->link);
+printf("data #%s#\n", prf->data);
+			if (prf->data)
+				free(prf->data);
+printf("link #%s#\n", prf->link);
+			if (prf->link)
+				free(prf->link);
 			free(prf);
 			prf = next;
 		}
 	}
+#endif
+
 
 	if (WIFEXITED(status)){
 		myexit(WEXITSTATUS(status));
