@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -246,6 +246,10 @@ void net_check_cfg(void) {
 	if (cfg.defaultgw)
 		check_default_gw(cfg.defaultgw);
 	else {
+		// if the first network has no assigned address,
+		// do not try to set up a gateway, because it will fail
+		if (cfg.bridge0.arg_ip_none)
+			return;
 		// first network is a regular bridge
 		if (cfg.bridge0.macvlan == 0)
 			cfg.defaultgw = cfg.bridge0.ip;
@@ -272,21 +276,8 @@ void net_dns_print(pid_t pid) {
 	// in case the pid is that of a firejail process, use the pid of the first child process
 	pid = switch_to_child(pid);
 
-	// now check if the pid belongs to a firejail sandbox
-	if (invalid_sandbox(pid)) {
-		fprintf(stderr, "Error: no valid sandbox\n");
-		exit(1);
-	}
-
-	// check privileges for non-root users
-	uid_t uid = getuid();
-	if (uid != 0) {
-		uid_t sandbox_uid = pid_get_uid(pid);
-		if (uid != sandbox_uid) {
-			fprintf(stderr, "Error: permission denied.\n");
-			exit(1);
-		}
-	}
+	// exit if no permission to join the sandbox
+	check_join_permission(pid);
 
 	EUID_ROOT();
 	if (join_namespace(pid, "mnt"))
