@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -89,7 +89,7 @@ int check_kernel_procs(void) {
 	// only user processes are available in /proc when running grsecurity
 	// EUID_ASSERT();
 
-	char *kern_proc[] = {
+	static char *kern_proc[] = {
 		"kthreadd",
 		"ksoftirqd",
 		"kworker",
@@ -204,14 +204,15 @@ void run_no_sandbox(int argc, char **argv) {
 			break;
 		}
 	}
-	// if shell is /usr/bin/firejail, replace it with /bin/bash
-	if (strcmp(cfg.shell, PATH_FIREJAIL) == 0) {
-		cfg.shell = "/bin/bash";
-		prog_index = 0;
-	}
+
+// if shell is /usr/bin/firejail, replace it with /bin/bash
+//	if (strcmp(cfg.shell, PATH_FIREJAIL) == 0) {
+//		cfg.shell = "/bin/bash";
+//		prog_index = 0;
+//	}
 
 	if (prog_index == 0) {
-		cfg.command_line = cfg.shell;
+		assert(cfg.command_line == NULL); // runs cfg.shell
 		cfg.window_title = cfg.shell;
 	} else {
 		build_cmdline(&cfg.command_line, &cfg.window_title, argc, argv, prog_index);
@@ -230,5 +231,5 @@ void run_no_sandbox(int argc, char **argv) {
 
 	arg_quiet = 1;
 
-	start_application(1, NULL);
+	start_application(1, -1, NULL);
 }
