@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -18,6 +18,8 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 #include "firejail.h"
+#include "../include/seccomp.h"
+#include "../include/syscall.h"
 #include <dirent.h>
 #include <sys/stat.h>
 extern char *xephyr_screen;
@@ -148,7 +150,11 @@ static int check_netoptions(void) {
 }
 
 static int check_nodbus(void) {
-	return arg_nodbus != 0;
+	return arg_dbus_user != DBUS_POLICY_ALLOW || arg_dbus_system != DBUS_POLICY_ALLOW;
+}
+
+static int check_nosound(void) {
+	return arg_nosound != 0;
 }
 
 static int check_x11(void) {
@@ -167,6 +173,7 @@ Cond conditionals[] = {
 	{"HAS_APPIMAGE", check_appimage},
 	{"HAS_NET", check_netoptions},
 	{"HAS_NODBUS", check_nodbus},
+	{"HAS_NOSOUND", check_nosound},
 	{"HAS_X11", check_x11},
 	{"BROWSER_DISABLE_U2F", check_disable_u2f},
 	{"BROWSER_ALLOW_DRM", check_allow_drm},
@@ -320,12 +327,10 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 	else if (strcmp(ptr, "seccomp") == 0) {
-#ifdef HAVE_SECCOMP
 		if (checkcfg(CFG_SECCOMP))
 			arg_seccomp = 1;
 		else
 			warning_feature_disabled("seccomp");
-#endif
 		return 0;
 	}
 	else if (strcmp(ptr, "caps") == 0) {
@@ -378,10 +383,12 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 	else if (strcmp(ptr, "private-cache") == 0) {
+#ifdef HAVE_USERTMPFS
 		if (checkcfg(CFG_PRIVATE_CACHE))
 			arg_private_cache = 1;
 		else
 			warning_feature_disabled("private-cache");
+#endif
 		return 0;
 	}
 	else if (strcmp(ptr, "private-dev") == 0) {
@@ -397,7 +404,13 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 	else if (strcmp(ptr, "nogroups") == 0) {
-		arg_nogroups = 1;
+		// nvidia cards require video group; disable nogroups
+		if (access("/dev/nvidiactl", R_OK) == 0 && arg_no3d == 0) {
+			fwarning("Warning: NVIDIA card detected, nogroups command disabled\n");
+			arg_nogroups = 0;
+		}
+		else
+			arg_nogroups = 1;
 		return 0;
 	}
 	else if (strcmp(ptr, "nosound") == 0) {
@@ -425,11 +438,148 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 	else if (strcmp(ptr, "nodbus") == 0) {
-		arg_nodbus = 1;
+#ifdef HAVE_DBUSPROXY
+		arg_dbus_user = DBUS_POLICY_BLOCK;
+		arg_dbus_system = DBUS_POLICY_BLOCK;
+#endif
 		return 0;
 	}
+	else if (strncmp("dbus-user ", ptr, 10) == 0) {
+#ifdef HAVE_DBUSPROXY
+		ptr += 10;
+		if (strcmp("filter", ptr) == 0) {
+			if (arg_dbus_user == DBUS_POLICY_BLOCK) {
+				fprintf(stderr, "Error: Cannot relax dbus-user policy, it is already set to block\n");
+			} else {
+				arg_dbus_user = DBUS_POLICY_FILTER;
+			}
+		} else if (strcmp("none", ptr) == 0) {
+			if (arg_dbus_log_user) {
+				fprintf(stderr, "Error: --dbus-user.log requires --dbus-user=filter\n");
+				exit(1);
+			}
+			arg_dbus_user = DBUS_POLICY_BLOCK;
+		} else {
+			fprintf(stderr, "Unknown dbus-user policy: %s\n", ptr);
+			exit(1);
+		}
+#endif
+		return 0;
+	}
+	else if (strncmp(ptr, "dbus-user.see ", 14) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_name(ptr + 14)) {
+			fprintf(stderr, "Invalid dbus-user.see name: %s\n", ptr + 15);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp(ptr, "dbus-user.talk ", 15) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_name(ptr + 15)) {
+			fprintf(stderr, "Error: Invalid dbus-user.talk name: %s\n", ptr + 15);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp(ptr, "dbus-user.own ", 14) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_name(ptr + 14)) {
+			fprintf(stderr, "Error: Invalid dbus-user.own name: %s\n", ptr + 14);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp(ptr, "dbus-user.call ", 15) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_call_rule(ptr + 15)) {
+			fprintf(stderr, "Error: Invalid dbus-user.call rule: %s\n", ptr + 15);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp(ptr, "dbus-user.broadcast ", 20) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_call_rule(ptr + 20)) {
+			fprintf(stderr, "Error: Invalid dbus-user.broadcast rule: %s\n", ptr + 20);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp("dbus-system ", ptr, 12) == 0) {
+#ifdef HAVE_DBUSPROXY
+		ptr += 12;
+		if (strcmp("filter", ptr) == 0) {
+			if (arg_dbus_system == DBUS_POLICY_BLOCK) {
+				fprintf(stderr, "Error: Cannot relax dbus-system policy, it is already set to block\n");
+			} else {
+				arg_dbus_system = DBUS_POLICY_FILTER;
+			}
+		} else if (strcmp("none", ptr) == 0) {
+			if (arg_dbus_log_system) {
+				fprintf(stderr, "Error: --dbus-system.log requires --dbus-system=filter\n");
+				exit(1);
+			}
+			arg_dbus_system = DBUS_POLICY_BLOCK;
+		} else {
+			fprintf(stderr, "Error: Unknown dbus-system policy: %s\n", ptr);
+			exit(1);
+		}
+#endif
+		return 0;
+	}
+	else if (strncmp(ptr, "dbus-system.see ", 16) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_name(ptr + 16)) {
+			fprintf(stderr, "Error: Invalid dbus-system.see name: %s\n", ptr + 17);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp(ptr, "dbus-system.talk ", 17) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_name(ptr + 17)) {
+			fprintf(stderr, "Error: Invalid dbus-system.talk name: %s\n", ptr + 17);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp(ptr, "dbus-system.own ", 16) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_name(ptr + 16)) {
+			fprintf(stderr, "Error: Invalid dbus-system.own name: %s\n", ptr + 16);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp(ptr, "dbus-system.call ", 17) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_call_rule(ptr + 17)) {
+			fprintf(stderr, "Error: Invalid dbus-system.call rule: %s\n", ptr + 17);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
+	else if (strncmp(ptr, "dbus-system.broadcast ", 22) == 0) {
+#ifdef HAVE_DBUSPROXY
+		if (!dbus_check_call_rule(ptr + 22)) {
+			fprintf(stderr, "Error: Invalid dbus-system.broadcast rule: %s\n", ptr + 22);
+			exit(1);
+		}
+#endif
+		return 1;
+	}
 	else if (strcmp(ptr, "nou2f") == 0) {
-	        arg_nou2f = 1;
+		arg_nou2f = 1;
 		return 0;
 	}
 	else if (strcmp(ptr, "netfilter") == 0) {
@@ -466,6 +616,17 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		}
 		else
 			warning_feature_disabled("networking");
+#endif
+		return 0;
+	}
+	else if (strncmp(ptr, "netns  ", 6) == 0) {
+#ifdef HAVE_NETWORK
+		if (checkcfg(CFG_NETWORK)) {
+			arg_netns = ptr + 6;
+			check_netns(arg_netns);
+		}
+		else
+			warning_feature_disabled("networking");
 #endif
 		return 0;
 	}
@@ -595,6 +756,12 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 				fprintf(stderr, "Error: invalid MAC address\n");
 				exit(1);
 			}
+
+			// check multicast address
+			if (br->macsandbox[0] & 1) {
+				fprintf(stderr, "Error: invalid MAC address (multicast)\n");
+				exit(1);
+			}
 		}
 		else
 			warning_feature_disabled("networking");
@@ -672,7 +839,10 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 			// configure this IP address for the last bridge defined
 			if (strcmp(ptr + 3, "none") == 0)
 				br->arg_ip_none = 1;
-			else {
+			else if (strcmp(ptr + 3, "dhcp") == 0) {
+				br->arg_ip_none = 1;
+				br->arg_ip_dhcp = 1;
+			} else {
 				if (atoip(ptr + 3, &br->ipsandbox)) {
 					fprintf(stderr, "Error: invalid IP address\n");
 					exit(1);
@@ -693,21 +863,24 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 				fprintf(stderr, "Error: no network device configured\n");
 				exit(1);
 			}
-			if (br->ip6sandbox) {
+			if (br->arg_ip6_dhcp || br->ip6sandbox) {
 				fprintf(stderr, "Error: cannot configure the IP address twice for the same interface\n");
 				exit(1);
 			}
 
 			// configure this IP address for the last bridge defined
-			if (check_ip46_address(ptr + 4) == 0) {
-				fprintf(stderr, "Error: invalid IPv6 address\n");
-				exit(1);
+			if (strcmp(ptr + 4, "dhcp") == 0)
+				br->arg_ip6_dhcp = 1;
+			else {
+				if (check_ip46_address(ptr + 4) == 0) {
+					fprintf(stderr, "Error: invalid IPv6 address\n");
+					exit(1);
+				}
+
+				br->ip6sandbox = strdup(ptr + 4);
+				if (br->ip6sandbox == NULL)
+					errExit("strdup");
 			}
-
-			br->ip6sandbox = strdup(ptr + 4);
-			if (br->ip6sandbox == NULL)
-				errExit("strdup");
-
 		}
 		else
 			warning_feature_disabled("networking");
@@ -737,10 +910,9 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 	}
 
 	if (strncmp(ptr, "protocol ", 9) == 0) {
-#ifdef HAVE_SECCOMP
 		if (checkcfg(CFG_SECCOMP)) {
 			if (cfg.protocol) {
-				fwarning("two protocol lists are present, \"%s\" will be installed\n", cfg.protocol);
+				fwarning("more than one protocol list is present, \"%s\" will be installed\n", cfg.protocol);
 				return 0;
 			}
 
@@ -751,7 +923,6 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		}
 		else
 			warning_feature_disabled("seccomp");
-#endif
 		return 0;
 	}
 
@@ -760,68 +931,114 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 	if (strncmp(ptr, "rmenv ", 6) == 0) {
+		unsetenv(ptr + 6); // Remove also immediately from Firejail itself
 		env_store(ptr + 6, RMENV);
 		return 0;
 	}
 
 	// seccomp drop list on top of default list
 	if (strncmp(ptr, "seccomp ", 8) == 0) {
-#ifdef HAVE_SECCOMP
 		if (checkcfg(CFG_SECCOMP)) {
 			arg_seccomp = 1;
 			cfg.seccomp_list = seccomp_check_list(ptr + 8);
 		}
 		else if (!arg_quiet)
 			warning_feature_disabled("seccomp");
-#endif
+
+		return 0;
+	}
+	if (strncmp(ptr, "seccomp.32 ", 11) == 0) {
+		if (checkcfg(CFG_SECCOMP)) {
+			arg_seccomp32 = 1;
+			cfg.seccomp_list32 = seccomp_check_list(ptr + 11);
+		}
+		else if (!arg_quiet)
+			warning_feature_disabled("seccomp");
 
 		return 0;
 	}
 
 	if (strcmp(ptr, "seccomp.block-secondary") == 0) {
-#ifdef HAVE_SECCOMP
 		if (checkcfg(CFG_SECCOMP)) {
 			arg_seccomp_block_secondary = 1;
 		}
 		else
 			warning_feature_disabled("seccomp");
-#endif
 		return 0;
 	}
 	// seccomp drop list without default list
 	if (strncmp(ptr, "seccomp.drop ", 13) == 0) {
-#ifdef HAVE_SECCOMP
 		if (checkcfg(CFG_SECCOMP)) {
 			arg_seccomp = 1;
 			cfg.seccomp_list_drop = seccomp_check_list(ptr + 13);
 		}
 		else
 			warning_feature_disabled("seccomp");
-#endif
+		return 0;
+	}
+	if (strncmp(ptr, "seccomp.32.drop ", 13) == 0) {
+		if (checkcfg(CFG_SECCOMP)) {
+			arg_seccomp32 = 1;
+			cfg.seccomp_list_drop32 = seccomp_check_list(ptr + 13);
+		}
+		else
+			warning_feature_disabled("seccomp");
 		return 0;
 	}
 
 	// seccomp keep list
 	if (strncmp(ptr, "seccomp.keep ", 13) == 0) {
-#ifdef HAVE_SECCOMP
 		if (checkcfg(CFG_SECCOMP)) {
 			arg_seccomp = 1;
 			cfg.seccomp_list_keep= seccomp_check_list(ptr + 13);
 		}
 		else
 			warning_feature_disabled("seccomp");
-#endif
+		return 0;
+	}
+	if (strncmp(ptr, "seccomp.32.keep ", 13) == 0) {
+		if (checkcfg(CFG_SECCOMP)) {
+			arg_seccomp32 = 1;
+			cfg.seccomp_list_keep32 = seccomp_check_list(ptr + 13);
+		}
+		else
+			warning_feature_disabled("seccomp");
 		return 0;
 	}
 
 	// memory deny write&execute
 	if (strcmp(ptr, "memory-deny-write-execute") == 0) {
-#ifdef HAVE_SECCOMP
 		if (checkcfg(CFG_SECCOMP))
 			arg_memory_deny_write_execute = 1;
 		else
 			warning_feature_disabled("seccomp");
-#endif
+		return 0;
+	}
+
+	// seccomp error action
+	if (strncmp(ptr, "seccomp-error-action ", 21) == 0) {
+		if (checkcfg(CFG_SECCOMP)) {
+			int config_seccomp_error_action = checkcfg(CFG_SECCOMP_ERROR_ACTION);
+			if (config_seccomp_error_action == -1) {
+				if (strcmp(ptr + 21, "kill") == 0)
+					arg_seccomp_error_action = SECCOMP_RET_KILL;
+				else if (strcmp(ptr + 21, "log") == 0)
+					arg_seccomp_error_action = SECCOMP_RET_LOG;
+				else {
+					arg_seccomp_error_action = errno_find_name(ptr + 21);
+					if (arg_seccomp_error_action == -1)
+						errExit("seccomp-error-action: unknown errno");
+				}
+				cfg.seccomp_error_action = strdup(ptr + 21);
+				if (!cfg.seccomp_error_action)
+					errExit("strdup");
+			} else {
+				arg_seccomp_error_action = config_seccomp_error_action;
+				cfg.seccomp_error_action = config_seccomp_error_action_str;
+				warning_feature_disabled("seccomp-error-action");
+			}
+		} else
+			warning_feature_disabled("seccomp");
 		return 0;
 	}
 
@@ -1214,12 +1431,12 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 	// filesystem bind
 	if (strncmp(ptr, "bind ", 5) == 0) {
 		if (checkcfg(CFG_BIND)) {
+			// extract two directories
 			if (getuid() != 0) {
 				fprintf(stderr, "Error: --bind option is available only if running as root\n");
 				exit(1);
 			}
 
-			// extract two directories
 			char *dname1 = ptr + 5;
 			char *dname2 = split_comma(dname1); // this inserts a '0 to separate the two dierctories
 			if (dname2 == NULL) {
@@ -1281,7 +1498,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 			arg_rlimit_as = 1;
 		}
 		else {
-			fprintf(stderr, "Invalid rlimit option on line %d\n", lineno);
+			fprintf(stderr, "Error: Invalid rlimit option on line %d\n", lineno);
 			exit(1);
 		}
 
@@ -1297,7 +1514,10 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		if (checkcfg(CFG_JOIN) || getuid() == 0) {
 			// try to join by name only
 			pid_t pid;
-			if (!name2pid(ptr + 14, &pid)) {
+			EUID_ROOT();
+			int r = name2pid(ptr + 14, &pid);
+			EUID_USER();
+			if (!r) {
 				if (!cfg.shell && !arg_shell_none)
 					cfg.shell = guess_shell();
 
@@ -1365,10 +1585,12 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 	else if (strncmp(ptr, "noexec ", 7) == 0)
 		ptr += 7;
 	else if (strncmp(ptr, "tmpfs ", 6) == 0) {
+#ifndef HAVE_USERTMPFS
 		if (getuid() != 0) {
 			fprintf(stderr, "Error: tmpfs available only when running the sandbox as root\n");
 			exit(1);
 		}
+#endif
 		ptr += 6;
 	}
 	else {
@@ -1435,17 +1657,18 @@ void profile_read(const char *fname) {
 		exit(1);
 	}
 	if (access(fname, R_OK)) {
+		int errsv = errno;
 		// if the file ends in ".local", do not exit
 		const char *base = gnu_basename(fname);
 		char *ptr = strstr(base, ".local");
-		if (ptr && strlen(ptr) == 6)
+		if (ptr && strlen(ptr) == 6 && errsv != EACCES)
 			return;
 
 		fprintf(stderr, "Error: cannot access profile file: %s\n", fname);
 		exit(1);
 	}
 
-	// allow debuggers
+	// --allow-debuggers - skip disable-devel.inc file
 	if (arg_allow_debuggers) {
 		char *tmp = strrchr(fname, '/');
 		if (tmp && *(tmp + 1) != '\0') {
@@ -1454,6 +1677,15 @@ void profile_read(const char *fname) {
 				return;
 		}
 	}
+	// --appimage - skip disable-shell.inc file
+	if (arg_appimage) {
+		char *tmp = strrchr(fname, '/');
+		if (tmp && *(tmp + 1) != '\0') {
+			tmp++;
+			if (strcmp(tmp, "disable-shell.inc") == 0)
+				return;
+		}
+	}
 
 	// open profile file:
 	FILE *fp = fopen(fname, "r");
@@ -1500,7 +1732,7 @@ void profile_read(const char *fname) {
 		}
 
 		// process include
-		if (strncmp(ptr, "include ", 8) == 0) {
+		if (strncmp(ptr, "include ", 8) == 0 && !is_in_ignore_list(ptr)) {
 			include_level++;
 
 			// expand macros in front of the include profile file
