@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -19,16 +19,18 @@
 */
 
 #include "firejail.h"
+#include "../include/seccomp.h"
+#include <sys/mman.h>
 #include <sys/mount.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
-#include <sys/prctl.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/types.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <syscall.h>
 
 #include <sched.h>
 #ifndef CLONE_NEWUSER
@@ -37,16 +39,15 @@
 
 #include <sys/prctl.h>
 #ifndef PR_SET_NO_NEW_PRIVS
-# define PR_SET_NO_NEW_PRIVS 38
+#define PR_SET_NO_NEW_PRIVS 38
 #endif
 #ifndef PR_GET_NO_NEW_PRIVS
-# define PR_GET_NO_NEW_PRIVS 39
+#define PR_GET_NO_NEW_PRIVS 39
 #endif
 
 #ifdef HAVE_APPARMOR
 #include <sys/apparmor.h>
 #endif
-#include <syscall.h>
 
 
 static int force_nonewprivs = 0;
@@ -125,6 +126,38 @@ static void set_caps(void) {
 		caps_drop_dac_override();
 }
 
+#ifdef HAVE_APPARMOR
+void set_apparmor(void) {
+	EUID_ASSERT();
+	if (checkcfg(CFG_APPARMOR) && arg_apparmor) {
+		if (aa_change_onexec("firejail-default")) {
+			fwarning("Cannot confine the application using AppArmor.\n"
+				"Maybe firejail-default AppArmor profile is not loaded into the kernel.\n"
+				"As root, run \"aa-enforce firejail-default\" to load it.\n");
+		}
+		else if (arg_debug)
+			printf("AppArmor enabled\n");
+	}
+}
+#endif
+
+static void seccomp_debug(void) {
+	if (arg_debug == 0)
+		return;
+
+	EUID_USER();
+	printf("Seccomp directory:\n");
+	ls(RUN_SECCOMP_DIR);
+	struct stat s;
+	if (stat(RUN_SECCOMP_LIST, &s) == 0) {
+		printf("Active seccomp files:\n");
+		cat(RUN_SECCOMP_LIST);
+	}
+	else
+		printf("No active seccomp files\n");
+	EUID_ROOT();
+}
+
 static void save_nogroups(void) {
 	if (arg_nogroups == 0)
 		return;
@@ -170,16 +203,17 @@ static void save_umask(void) {
 	}
 }
 
-static FILE *create_ready_for_join_file(void) {
-	FILE *fp = fopen(RUN_READY_FOR_JOIN, "wxe");
-	if (fp) {
-		ASSERT_PERMS_STREAM(fp, 0, 0, 0644);
-		return fp;
-	}
-	else {
-		fprintf(stderr, "Error: cannot create %s\n", RUN_READY_FOR_JOIN);
-		exit(1);
-	}
+static char *create_join_file(void) {
+	int fd = open(RUN_JOIN_FILE, O_RDWR|O_CREAT|O_EXCL|O_CLOEXEC, S_IRUSR | S_IWRITE | S_IRGRP | S_IROTH);
+	if (fd == -1)
+		errExit("open");
+	if (ftruncate(fd, 1) == -1)
+		errExit("ftruncate");
+	char *rv = mmap(NULL, 1, PROT_WRITE, MAP_SHARED, fd, 0);
+	if (rv == MAP_FAILED)
+		errExit("mmap");
+	close(fd);
+	return rv;
 }
 
 static void sandbox_if_up(Bridge *br) {
@@ -337,6 +371,8 @@ static int monitor_application(pid_t app_pid) {
 				continue;
 			if (pid == 1)
 				continue;
+			if ((pid_t) pid == dhclient4_pid || (pid_t) pid == dhclient6_pid)
+				continue;
 
 			// todo: make this generic
 			// Dillo browser leaves a dpid process running, we need to shut it down
@@ -364,19 +400,8 @@ static int monitor_application(pid_t app_pid) {
 }
 
 static void print_time(void) {
-	if (start_timestamp) {
-		unsigned long long end_timestamp = getticks();
-		// measure 1 ms
-		usleep(1000);
-		unsigned long long onems = getticks() - end_timestamp;
-		if (onems) {
-			fmessage("Child process initialized in %.02f ms\n",
-				(float) (end_timestamp - start_timestamp) / (float) onems);
-			return;
-		}
-	}
-
-	fmessage("Child process initialized\n");
+	float delta = timetrace_end();
+	fmessage("Child process initialized in %.02f ms\n", delta);
 }
 
 
@@ -436,7 +461,7 @@ static int ok_to_run(const char *program) {
 	return 0;
 }
 
-void start_application(int no_sandbox, FILE *fp) {
+void start_application(int no_sandbox, int fd, char *set_sandbox_status) {
 	// set environment
 	if (no_sandbox == 0) {
 		env_defaults();
@@ -446,7 +471,7 @@ void start_application(int no_sandbox, FILE *fp) {
 	umask(orig_umask);
 
 	if (arg_debug) {
-		printf("starting application\n");
+		printf("Starting application\n");
 		printf("LD_PRELOAD=%s\n", getenv("LD_PRELOAD"));
 	}
 
@@ -456,20 +481,13 @@ void start_application(int no_sandbox, FILE *fp) {
 	if (arg_audit) {
 		assert(arg_audit_prog);
 
-		if (fp) {
-			fprintf(fp, "ready\n");
-			fclose(fp);
-		}
 #ifdef HAVE_GCOV
 		__gcov_dump();
 #endif
-#ifdef HAVE_SECCOMP
 		seccomp_install_filters();
-#endif
+		if (set_sandbox_status)
+			*set_sandbox_status = SANDBOX_DONE;
 		execl(arg_audit_prog, arg_audit_prog, NULL);
-
-		perror("execl");
-		exit(1);
 	}
 	//****************************************
 	// start the program without using a shell
@@ -492,58 +510,56 @@ void start_application(int no_sandbox, FILE *fp) {
 		if (!arg_command && !arg_quiet)
 			print_time();
 
-		int rv = ok_to_run(cfg.original_argv[cfg.original_program_index]);
-
-		if (fp) {
-			fprintf(fp, "ready\n");
-			fclose(fp);
+		if (ok_to_run(cfg.original_argv[cfg.original_program_index]) == 0) {
+			fprintf(stderr, "Error: no suitable %s executable found\n", cfg.original_argv[cfg.original_program_index]);
+			exit(1);
 		}
+
 #ifdef HAVE_GCOV
 		__gcov_dump();
 #endif
-#ifdef HAVE_SECCOMP
 		seccomp_install_filters();
-#endif
-		if (rv)
-			execvp(cfg.original_argv[cfg.original_program_index], &cfg.original_argv[cfg.original_program_index]);
-		else
-			fprintf(stderr, "Error: no suitable %s executable found\n", cfg.original_argv[cfg.original_program_index]);
-		exit(1);
+
+		if (set_sandbox_status)
+			*set_sandbox_status = SANDBOX_DONE;
+		execvp(cfg.original_argv[cfg.original_program_index], &cfg.original_argv[cfg.original_program_index]);
 	}
 	//****************************************
 	// start the program using a shell
 	//****************************************
 	else {
 		assert(cfg.shell);
-		assert(cfg.command_line);
 
 		char *arg[5];
 		int index = 0;
 		arg[index++] = cfg.shell;
-		if (login_shell) {
-			arg[index++] = "-l";
-			if (arg_debug)
-				printf("Starting %s login shell\n", cfg.shell);
-		} else {
-			arg[index++] = "-c";
+		if (cfg.command_line) {
 			if (arg_debug)
 				printf("Running %s command through %s\n", cfg.command_line, cfg.shell);
+			arg[index++] = "-c";
 			if (arg_doubledash)
 				arg[index++] = "--";
 			arg[index++] = cfg.command_line;
 		}
-		arg[index] = NULL;
+		else if (login_shell) {
+			if (arg_debug)
+				printf("Starting %s login shell\n", cfg.shell);
+			arg[index++] = "-l";
+		}
+		else if (arg_debug)
+			printf("Starting %s shell\n", cfg.shell);
+
 		assert(index < 5);
+		arg[index] = NULL;
 
 		if (arg_debug) {
 			char *msg;
-			if (asprintf(&msg, "sandbox %d, execvp into %s", sandbox_pid, cfg.command_line) == -1)
+			if (asprintf(&msg, "sandbox %d, execvp into %s",
+				sandbox_pid, cfg.command_line ? cfg.command_line : cfg.shell) == -1)
 				errExit("asprintf");
 			logmsg(msg);
 			free(msg);
-		}
 
-		if (arg_debug) {
 			int i;
 			for (i = 0; i < 5; i++) {
 				if (arg[i] == NULL)
@@ -555,21 +571,22 @@ void start_application(int no_sandbox, FILE *fp) {
 		if (!arg_command && !arg_quiet)
 			print_time();
 
-		if (fp) {
-			fprintf(fp, "ready\n");
-			fclose(fp);
-		}
 #ifdef HAVE_GCOV
 		__gcov_dump();
 #endif
-#ifdef HAVE_SECCOMP
 		seccomp_install_filters();
-#endif
+
+		if (set_sandbox_status)
+			*set_sandbox_status = SANDBOX_DONE;
 		execvp(arg[0], arg);
+
+		// join sandbox without shell in the mount namespace
+		if (fd > -1)
+			fexecve(fd, arg, environ);
 	}
 
-	perror("execvp");
-	exit(1); // it should never get here!!!
+	perror("Cannot start application");
+	exit(1);
 }
 
 static void enforce_filters(void) {
@@ -626,6 +643,8 @@ int sandbox(void* sandbox_arg) {
 	if (mount(LIBDIR "/firejail", RUN_FIREJAIL_LIB_DIR, NULL, MS_BIND, NULL) < 0 ||
 	    mount(NULL, RUN_FIREJAIL_LIB_DIR, NULL, MS_RDONLY|MS_NOSUID|MS_NODEV|MS_BIND|MS_REMOUNT, NULL) < 0)
 		errExit("mounting " RUN_FIREJAIL_LIB_DIR);
+	// keep a copy of dhclient executable before the filesystem is modified
+	dhcp_store_exec();
 
 	//****************************
 	// log sandbox data
@@ -766,7 +785,6 @@ int sandbox(void* sandbox_arg) {
 	//  - build seccomp filters
 	//  - create an empty /etc/ld.so.preload
 	//****************************
-#ifdef HAVE_SECCOMP
 	if (cfg.protocol) {
 		if (arg_debug)
 			printf("Build protocol filter: %s\n", cfg.protocol);
@@ -777,9 +795,6 @@ int sandbox(void* sandbox_arg) {
 		if (rv)
 			exit(rv);
 	}
-	if (arg_seccomp && (cfg.seccomp_list || cfg.seccomp_list_drop || cfg.seccomp_list_keep))
-		arg_seccomp_postexec = 1;
-#endif
 
 	// need ld.so.preload if tracing or seccomp with any non-default lists
 	bool need_preload = arg_trace || arg_tracelog || arg_seccomp_postexec;
@@ -847,19 +862,6 @@ int sandbox(void* sandbox_arg) {
 	if (arg_private_dev)
 		fs_private_dev();
 
-	if (arg_private_etc) {
-		if (cfg.chrootdir)
-			fwarning("private-etc feature is disabled in chroot\n");
-		else if (arg_overlay)
-			fwarning("private-etc feature is disabled in overlay\n");
-		else {
-			fs_private_dir_list("/etc", RUN_ETC_DIR, cfg.etc_private_keep);
-			// create /etc/ld.so.preload file again
-			if (need_preload)
-				fs_trace_preload();
-		}
-	}
-
 	if (arg_private_opt) {
 		if (cfg.chrootdir)
 			fwarning("private-opt feature is disabled in chroot\n");
@@ -911,14 +913,13 @@ int sandbox(void* sandbox_arg) {
 		}
 	}
 
+#ifdef HAVE_USERTMPFS
 	if (arg_private_cache) {
-		if (cfg.chrootdir)
-			fwarning("private-cache feature is disabled in chroot\n");
-		else if (arg_overlay)
-			fwarning("private-cache feature is disabled in overlay\n");
-		else
-			fs_private_cache();
+		EUID_USER();
+		profile_add("tmpfs ${HOME}/.cache");
+		EUID_ROOT();
 	}
+#endif
 
 	if (arg_private_tmp) {
 		// private-tmp is implemented as a whitelist
@@ -930,9 +931,9 @@ int sandbox(void* sandbox_arg) {
 	//****************************
 	// Session D-BUS
 	//****************************
-	if (arg_nodbus)
-		dbus_disable();
-
+#ifdef HAVE_DBUSPROXY
+	dbus_apply_policy();
+#endif
 
 	//****************************
 	// hosts and hostname
@@ -962,6 +963,21 @@ int sandbox(void* sandbox_arg) {
 	else if (arg_disable_mnt)
 		fs_mnt(0);
 
+	// Install new /etc last, so we can use it as long as possible
+	if (arg_private_etc) {
+		if (cfg.chrootdir)
+			fwarning("private-etc feature is disabled in chroot\n");
+		else if (arg_overlay)
+			fwarning("private-etc feature is disabled in overlay\n");
+		else {
+			fs_private_dir_list("/etc", RUN_ETC_DIR, cfg.etc_private_keep);
+			fs_private_dir_list("/usr/etc", RUN_USR_ETC_DIR, cfg.etc_private_keep); // openSUSE
+			// create /etc/ld.so.preload file again
+			if (need_preload)
+				fs_trace_preload();
+		}
+	}
+
 	//****************************
 	// apply the profile file
 	//****************************
@@ -994,7 +1010,7 @@ int sandbox(void* sandbox_arg) {
 		fs_dev_disable_dvd();
 
 	if (arg_nou2f)
-	        fs_dev_disable_u2f();
+		fs_dev_disable_u2f();
 
 	if (arg_novideo)
 		fs_dev_disable_video();
@@ -1016,6 +1032,11 @@ int sandbox(void* sandbox_arg) {
 	fs_logger_print();
 	fs_logger_change_owner();
 
+	//****************************
+	// start dhcp client
+	//****************************
+	dhcp_start();
+
 	//****************************
 	// set application environment
 	//****************************
@@ -1072,7 +1093,6 @@ int sandbox(void* sandbox_arg) {
 	save_cgroup();
 
 	// set seccomp
-#ifdef HAVE_SECCOMP
 	// install protocol filter
 #ifdef SYS_socket
 	if (cfg.protocol) {
@@ -1090,39 +1110,41 @@ int sandbox(void* sandbox_arg) {
 	// if a keep list is available, disregard the drop list
 	if (arg_seccomp == 1) {
 		if (cfg.seccomp_list_keep)
-			seccomp_filter_keep();
+			seccomp_filter_keep(true);
 		else
-			seccomp_filter_drop();
-
+			seccomp_filter_drop(true);
 	}
-	else { // clean seccomp files under /run/firejail/mnt
-		int rv = unlink(RUN_SECCOMP_CFG);
-		rv |= unlink(RUN_SECCOMP_32);
-		(void) rv;
+	if (arg_seccomp32 == 1) {
+		if (cfg.seccomp_list_keep32)
+			seccomp_filter_keep(false);
+		else
+			seccomp_filter_drop(false);
+
 	}
 
 	if (arg_memory_deny_write_execute) {
+		if (arg_seccomp_error_action != EPERM) {
+			seccomp_filter_mdwx(true);
+			seccomp_filter_mdwx(false);
+		}
 		if (arg_debug)
 			printf("Install memory write&execute filter\n");
 		seccomp_load(RUN_SECCOMP_MDWX);	// install filter
+		seccomp_load(RUN_SECCOMP_MDWX_32);
 	}
-	else {
-		int rv = unlink(RUN_SECCOMP_MDWX);
-		(void) rv;
-	}
+
 	// make seccomp filters read-only
 	fs_remount(RUN_SECCOMP_DIR, MOUNT_READONLY, 0);
-#endif
+	seccomp_debug();
 
 	// set capabilities
 	set_caps();
 
 	//****************************************
-	// communicate progress of sandbox set up
-	// to --join
+	// relay status information to join option
 	//****************************************
 
-	FILE *rj = create_ready_for_join_file();
+	char *set_sandbox_status = create_join_file();
 
 	//****************************************
 	// create a new user namespace
@@ -1195,26 +1217,19 @@ int sandbox(void* sandbox_arg) {
 
 	if (app_pid == 0) {
 #ifdef HAVE_APPARMOR
-		if (checkcfg(CFG_APPARMOR) && arg_apparmor) {
-			errno = 0;
-			if (aa_change_onexec("firejail-default")) {
-				fwarning("Cannot confine the application using AppArmor.\n"
-					"Maybe firejail-default AppArmor profile is not loaded into the kernel.\n"
-					"As root, run \"aa-enforce firejail-default\" to load it.\n");
-			}
-			else if (arg_debug)
-				printf("AppArmor enabled\n");
-		}
+		// add apparmor confinement after the execve
+		set_apparmor();
 #endif
+
 		// set nice and rlimits
 		if (arg_nice)
 			set_nice(cfg.nice);
 		set_rlimits();
 
-		start_application(0, rj);
+		start_application(0, -1, set_sandbox_status);
 	}
 
-	fclose(rj);
+	munmap(set_sandbox_status, 1);
 
 	int status = monitor_application(app_pid);	// monitor application
 	flush_stdin();
