@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -18,7 +18,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifdef HAVE_SECCOMP
 #include "firejail.h"
 #include "../include/seccomp.h"
 #include <sys/mman.h>
@@ -191,15 +190,25 @@ static void seccomp_filter_block_secondary(void) {
 }
 
 // drop filter for seccomp option
-int seccomp_filter_drop(void) {
+int seccomp_filter_drop(bool native) {
+	const char *filter, *postexec_filter;
+
+	if (native) {
+		filter = RUN_SECCOMP_CFG;
+		postexec_filter = RUN_SECCOMP_POSTEXEC;
+	} else {
+		filter = RUN_SECCOMP_32;
+		postexec_filter = RUN_SECCOMP_POSTEXEC_32;
+	}
+
 	// if we have multiple seccomp commands, only one of them is executed
 	// in the following order:
 	//	- seccomp.drop list
 	//	- seccomp list
 	//	- seccomp
 	if (cfg.seccomp_list_drop == NULL) {
-		// default seccomp
-		if (cfg.seccomp_list == NULL) {
+		// default seccomp if error action is not changed
+		if (cfg.seccomp_list == NULL && cfg.seccomp_error_action) {
 			if (arg_seccomp_block_secondary)
 				seccomp_filter_block_secondary();
 			else {
@@ -224,19 +233,30 @@ int seccomp_filter_drop(void) {
 			if (arg_debug)
 				printf("Build default+drop seccomp filter\n");
 
+			const char *command, *list;
+			if (native) {
+				command = "default";
+				list = cfg.seccomp_list;
+			} else {
+				command = "default32";
+				list = cfg.seccomp_list32;
+			}
+
+			if (list == NULL)
+				list = "";
 			// build the seccomp filter as a regular user
 			int rv;
 			if (arg_allow_debuggers)
 				rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 7,
-					      PATH_FSECCOMP, "default", "drop", RUN_SECCOMP_CFG, RUN_SECCOMP_POSTEXEC, cfg.seccomp_list, "allow-debuggers");
+					      PATH_FSECCOMP, command, "drop", filter, postexec_filter, list, "allow-debuggers");
 			else
 				rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 6,
-					      PATH_FSECCOMP, "default", "drop", RUN_SECCOMP_CFG, RUN_SECCOMP_POSTEXEC, cfg.seccomp_list);
+					      PATH_FSECCOMP, command, "drop", filter, postexec_filter, list);
 			if (rv)
 				exit(rv);
 
 			// optimize the new filter
-			rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2, PATH_FSEC_OPTIMIZE, RUN_SECCOMP_CFG);
+			rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2, PATH_FSEC_OPTIMIZE, filter);
 			if (rv)
 				exit(rv);
 		}
@@ -250,36 +270,45 @@ int seccomp_filter_drop(void) {
 		if (arg_debug)
 			printf("Build drop seccomp filter\n");
 
+		const char *command, *list;
+		if (native) {
+			command = "drop";
+			list = cfg.seccomp_list_drop;
+		} else {
+			command = "drop32";
+			list = cfg.seccomp_list_drop32;
+		}
+
 		// build the seccomp filter as a regular user
 		int rv;
 		if (arg_allow_debuggers)
 			rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 6,
-				      PATH_FSECCOMP, "drop", RUN_SECCOMP_CFG, RUN_SECCOMP_POSTEXEC, cfg.seccomp_list_drop,  "allow-debuggers");
+				      PATH_FSECCOMP, command, filter, postexec_filter, list,  "allow-debuggers");
 		else
 			rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 5,
-				PATH_FSECCOMP, "drop", RUN_SECCOMP_CFG, RUN_SECCOMP_POSTEXEC, cfg.seccomp_list_drop);
+				      PATH_FSECCOMP, command, filter, postexec_filter, list);
 
 		if (rv)
 			exit(rv);
 
 		// optimize the drop filter
-		rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2, PATH_FSEC_OPTIMIZE, RUN_SECCOMP_CFG);
+		rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2, PATH_FSEC_OPTIMIZE, filter);
 		if (rv)
 			exit(rv);
 	}
 
 	// load the filter
-	if (seccomp_load(RUN_SECCOMP_CFG) == 0) {
+	if (seccomp_load(filter) == 0) {
 		if (arg_debug)
 			printf("seccomp filter configured\n");
 	}
 
 	if (arg_debug && access(PATH_FSEC_PRINT, X_OK) == 0) {
 		struct stat st;
-		if (stat(RUN_SECCOMP_POSTEXEC, &st) != -1 && st.st_size != 0) {
-			printf("configuring postexec seccomp filter in %s\n", RUN_SECCOMP_POSTEXEC);
+		if (stat(postexec_filter, &st) != -1 && st.st_size != 0) {
+			printf("configuring postexec seccomp filter in %s\n", postexec_filter);
 			sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2,
-				  PATH_FSEC_PRINT, RUN_SECCOMP_POSTEXEC);
+				  PATH_FSEC_PRINT, postexec_filter);
 		}
 	}
 
@@ -287,7 +316,7 @@ int seccomp_filter_drop(void) {
 }
 
 // keep filter for seccomp option
-int seccomp_filter_keep(void) {
+int seccomp_filter_keep(bool native) {
 	// secondary filters are not installed except when secondary
 	// architectures are explicitly blocked
 	if (arg_seccomp_block_secondary)
@@ -296,9 +325,20 @@ int seccomp_filter_keep(void) {
 	if (arg_debug)
 		printf("Build keep seccomp filter\n");
 
+	const char *filter, *postexec_filter, *list;
+	if (native) {
+		filter = RUN_SECCOMP_CFG;
+		postexec_filter = RUN_SECCOMP_POSTEXEC;
+		list = cfg.seccomp_list_keep;
+	} else {
+		filter = RUN_SECCOMP_32;
+		postexec_filter = RUN_SECCOMP_POSTEXEC_32;
+		list = cfg.seccomp_list_keep32;
+	}
+
 	// build the seccomp filter as a regular user
 	int rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 5,
-		 PATH_FSECCOMP, "keep", RUN_SECCOMP_CFG, RUN_SECCOMP_POSTEXEC, cfg.seccomp_list_keep);
+		 PATH_FSECCOMP, "keep", filter, postexec_filter, list);
 
 	if (rv) {
 		fprintf(stderr, "Error: cannot configure seccomp filter\n");
@@ -309,44 +349,60 @@ int seccomp_filter_keep(void) {
 		printf("seccomp filter configured\n");
 
 	// load the filter
-	if (seccomp_load(RUN_SECCOMP_CFG) == 0) {
+	if (seccomp_load(filter) == 0) {
 		if (arg_debug)
 			printf("seccomp filter configured\n");
 	}
 
 	if (arg_debug && access(PATH_FSEC_PRINT, X_OK) == 0) {
 		struct stat st;
-		if (stat(RUN_SECCOMP_POSTEXEC, &st) != -1 && st.st_size != 0) {
-			printf("configuring postexec seccomp filter in %s\n", RUN_SECCOMP_POSTEXEC);
+		if (stat(postexec_filter, &st) != -1 && st.st_size != 0) {
+			printf("configuring postexec seccomp filter in %s\n", postexec_filter);
 			sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 2,
-				  PATH_FSEC_PRINT, RUN_SECCOMP_POSTEXEC);
+				  PATH_FSEC_PRINT, postexec_filter);
 		}
 	}
 
 	return 0;
 }
 
+// create mdwx filter for non-default error action
+int seccomp_filter_mdwx(bool native) {
+	if (arg_debug)
+		printf("Build memory-deny-write-execute filter\n");
+
+	const char *command, *filter;
+	if (native) {
+		command = "memory-deny-write-execute";
+		filter = RUN_SECCOMP_MDWX;
+	} else {
+		command = "memory-deny-write-execute.32";
+		filter = RUN_SECCOMP_MDWX_32;
+	}
+
+	// build the seccomp filter as a regular user
+	int rv = sbox_run(SBOX_USER | SBOX_CAPS_NONE | SBOX_SECCOMP, 3,
+		 PATH_FSECCOMP, command, filter);
+
+	if (rv) {
+		fprintf(stderr, "Error: cannot build memory-deny-write-execute filter\n");
+		exit(rv);
+	}
+
+	if (arg_debug)
+		printf("Memory-deny-write-execute filter configured\n");
+
+	return 0;
+}
+
 void seccomp_print_filter(pid_t pid) {
 	EUID_ASSERT();
 
 	// in case the pid is that of a firejail process, use the pid of the first child process
 	pid = switch_to_child(pid);
 
-	// now check if the pid belongs to a firejail sandbox
-	if (invalid_sandbox(pid)) {
-		fprintf(stderr, "Error: no valid sandbox\n");
-		exit(1);
-	}
-
-	// check privileges for non-root users
-	uid_t uid = getuid();
-	if (uid != 0) {
-		uid_t sandbox_uid = pid_get_uid(pid);
-		if (uid != sandbox_uid) {
-			fprintf(stderr, "Error: permission denied.\n");
-			exit(1);
-		}
-	}
+	// exit if no permission to join the sandbox
+	check_join_permission(pid);
 
 	// find the seccomp list file
 	EUID_ROOT();
@@ -388,5 +444,3 @@ errexit:
 	printf("Cannot access seccomp filter.\n");
 	exit(1);
 }
-
-#endif // HAVE_SECCOMP
