@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -53,7 +53,7 @@ void caps(pid_t pid, int print_procs) {
 	for (i = 0; i < max_pids; i++) {
 		if (pids[i].level == 1) {
 			if (print_procs || pid == 0)
-				pid_print_list(i, arg_nowrap);
+				pid_print_list(i, arg_wrap);
 			int child = find_child(i);
 			if (child != -1)
 				print_caps(child);
