@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -28,6 +28,6 @@ void list(void) {
 		if (i == skip_process)
 			continue;
 		if (pids[i].level == 1)
-			pid_print_list(i, arg_nowrap);
+			pid_print_list(i, arg_wrap);
 	}
 }
