@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -28,7 +28,7 @@ void tree(pid_t pid) {
 		if (i == skip_process)
 			continue;
 		if (pids[i].level == 1)
-			pid_print_tree(i, 0, arg_nowrap);
+			pid_print_tree(i, 0, arg_wrap);
 	}
 	printf("\n");
 }
