@@ -1,5 +1,5 @@
  /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -47,6 +47,7 @@ static void usage(void) {
 	printf("\tfnet config mac addr\n");
 	printf("\tfnet config ipv6 dev ip\n");
 	printf("\tfnet ifup dev\n");
+  printf("\tfnet waitll dev\n");
 }
 
 int main(int argc, char **argv) {
@@ -63,16 +64,18 @@ printf("\n");
 		usage();
 		return 1;
 	}
+	if (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0 || strcmp(argv[1], "-?") ==0) {
+		usage();
+		return 0;
+	}
+
+	warn_dumpable();
 
 	char *quiet = getenv("FIREJAIL_QUIET");
 	if (quiet && strcmp(quiet, "yes") == 0)
 		arg_quiet = 1;
 
-	if (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0 || strcmp(argv[1], "-?") ==0) {
-		usage();
-		return 0;
-	}
-	else if (argc == 3 && strcmp(argv[1], "ifup") == 0) {
+	if (argc == 3 && strcmp(argv[1], "ifup") == 0) {
 		net_if_up(argv[2]);
 	}
 	else if (argc == 2 && strcmp(argv[1], "printif") == 0) {
@@ -141,6 +144,9 @@ printf("\n");
 	else if (argc == 5 && strcmp(argv[1], "config") == 0 && strcmp(argv[2], "ipv6") == 0) {
 		net_if_ip6(argv[3], argv[4]);
 	}
+  else if (argc == 3 && strcmp(argv[1], "waitll") == 0) {
+    net_if_waitll(argv[2]);
+  }
 	else {
 		fprintf(stderr, "Error fnet: invalid arguments\n");
 		return 1;
