@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -24,6 +24,23 @@ static void usage(void) {
 	printf("\tfsec-print file - disassemble seccomp filter\n");
 }
 
+int arg_quiet = 0;
+void filter_add_errno(int fd, int syscall, int arg, void *ptrarg, bool native) {
+	(void) fd;
+	(void) syscall;
+	(void) arg;
+	(void) ptrarg;
+	(void) native;
+}
+
+void filter_add_blacklist_override(int fd, int syscall, int arg, void *ptrarg, bool native) {
+	(void) fd;
+	(void) syscall;
+	(void) arg;
+	(void) ptrarg;
+	(void) native;
+}
+
 int main(int argc, char **argv) {
 #if 0
 {
@@ -44,6 +61,8 @@ printf("\n");
 		return 0;
 	}
 
+	warn_dumpable();
+
 	char *fname = argv[1];
 
 	// open input file
