@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -19,7 +19,7 @@
  *
  *
  *
- * Parts of this code was lifted from libseccomp project, license LGPV 2.1.
+ * Parts of this code was lifted from libseccomp project, license LGPL 2.1.
  * This is the original copyright notice in libseccomp code:
  *
  *
@@ -250,7 +250,7 @@ static void bpf_decode_args(const struct sock_filter *bpf, unsigned int line) {
 		break;
 	case BPF_JMP:
 		if (BPF_OP(bpf->code) == BPF_JA) {
-			printf("%.4u", (line + 1) + bpf->k);
+			printf("%.4x", (line + 1) + bpf->k);
 		}
 		else {
 			const char *name = NULL;
