@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2019 Firejail Authors
+ * Copyright (C) 2014-2020 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -43,8 +43,11 @@ static void init_uid_gid_min(void) {
 
 	// read the real values from login.def
 	FILE *fp = fopen("/etc/login.defs", "r");
-	if (!fp)
-		goto errexit;
+	if (!fp) {
+		fp = fopen("/usr/etc/login.defs", "r"); // openSUSE
+		if (!fp)
+			goto errexit;
+	}
 
 	char buf[MAXBUF];
 	while (fgets(buf, MAXBUF, fp)) {
