@@ -1,6 +1,6 @@
 #!/bin/bash
 # This file is part of Firejail project
-# Copyright (C) 2014-2019 Firejail Authors
+# Copyright (C) 2014-2020 Firejail Authors
 # License GPL v2
 
 export MALLOC_CHECK_=3
@@ -25,6 +25,15 @@ else
 	echo "TESTING SKIP: transmission-gtk not found"
 fi
 
+which transmission-qt 2>/dev/null
+if [ "$?" -eq 0 ];
+then
+	echo "TESTING: transmission-qt x11 xorg"
+	./transmission-qt.exp
+else
+	echo "TESTING SKIP: transmission-qt not found"
+fi
+
 which thunderbird 2>/dev/null
 if [ "$?" -eq 0 ];
 then
