@@ -1,3 +1,7 @@
+// This file is part of Firejail project
+// Copyright (C) 2014-2020 Firejail Authors
+// License GPL v2
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -61,7 +65,7 @@ int main(int argc, char **argv) {
 		}
 
 		void *p = mmap (0, size, PROT_READ, MAP_SHARED, fd, 0);
-		if (!p) {
+		if (p == MAP_FAILED) {
 			fprintf(stderr, "TESTING ERROR: cannot map file for mprotect test\n");
 			return 1;
 		}
